<?php

	# It initially comes through as a POST, we now convert it to GET to make it prettier and allow pagination
	# This gives us a chance to strip out bad chars (we couldn't if we went straight to GET)
	if ($_POST)
	{
		$keywords = $_POST['keywords'];
		$keywords = str_replace('"', '', $keywords);
		$keywords = urlencode($keywords);
		redirect(URL_SITE.PAGENAME.'/'.$keywords.'/');
	}
	
	$searchpage = getPage(PAGENAME, 'pagename');
	$breadcrumb = getBreadcrumb($pagetype, $searchpage);
	
	# Setup defaults
	$defaultperpage = getTemplateOption('PRODUCT_LISTING_QUANTITY');
	if (!$defaultperpage) $defaultperpage = PER_PAGE_PRODUCTSBYCATEGORY;
	
	# Filter details
	$filtersql = getProductFilterSQL();
	$currentproductfilters = getSessionProductFilters();
	$sidebarfilters = getFilterOptions(true);
	$stocksql = (PRODUCT_OFFERS_WITHOUT == 1)? '' : 'AND offers > 0';
			
	# Sort out GET vars
	$keywords						= $db->cleanValue($_GET['var1']);
	$pagination['currentpage']		= ((isset($_GET['var2']) && is_numeric($_GET['var2'])) ? $_GET['var2'] : 1);
	$pagination['currentsort']		= (isset($_GET['var3']) ? $_GET['var3'] : 'relevance');
	$pagination['currentperpage']	= (isset($_GET['var4']) ? $_GET['var4'] : $defaultperpage);
	
	# Record this search
	if (!ADMIN_LOGGED_IN && $keywords != '')
	{
		$db->insert('searches', array('keywords' => $keywords, 'timestamp' => 'NOW()', 'ip' => $_SERVER['REMOTE_ADDR']));
	}
	
	# Count results for pagination
	$rs = $db->query("SELECT COUNT(*) FROM products p
		WHERE MATCH (description, keyfeatures, title) AGAINST ('$keywords' IN BOOLEAN MODE)
		AND enabled = 1
		AND hidden = 0
		AND parent_id = 0
		{$stocksql}
		AND ".implode('AND ', $filtersql));
	
	$pagination['quantity'] = $rs->getSingleResult();
	$pagination['pages'] = ceil($pagination['quantity']/$pagination['currentperpage']);
	$pagination['baselink'] = URL_SITE.PAGENAME.'/'.urlencode($keywords).'/';
	
	# Sort options
	$pagination['sortoptions']['relevance']['displayname'] = 'Relevance';
	$pagination['sortoptions']['relevance']['sql'] = 'searchrelevance DESC';
	$pagination['sortoptions']['price-asc']['displayname'] = 'Price (Low to High)';
	$pagination['sortoptions']['price-asc']['sql'] = 'price ASC';
	$pagination['sortoptions']['price-desc']['displayname'] = 'Price (High to Low)';
	$pagination['sortoptions']['price-desc']['sql'] = 'price DESC';
	$pagination['sortoptions']['reviewscore']['displayname'] = 'Customer Review Score';
	$pagination['sortoptions']['reviewscore']['sql'] = 'averagescore DESC';
	
	$orderby = $pagination['sortoptions'][$pagination['currentsort']]['sql'];

	# LIMIT
	$limitstart = ($pagination['currentpage'] - 1) * $pagination['currentperpage'];
	
	# Perform the search
	$sql = "SELECT *, MATCH (description, keyfeatures, title) AGAINST ('$keywords') as searchrelevance FROM products p
		WHERE MATCH (description, keyfeatures, title) AGAINST ('$keywords' IN BOOLEAN MODE)
		AND enabled = 1
		AND hidden = 0
		AND parent_id = 0
		{$stocksql} 
		AND ".implode('AND ', $filtersql)."
		ORDER BY $orderby
		LIMIT $limitstart, ".$pagination['currentperpage'];
	$rs = $db->query($sql);
	$ra = $rs->getFullArray();
	
	# Redirect if no results and not on page 1
	if ($pagination['currentpage'] <> 1 && !$ra)
	{	
		$redirectto = buildListingLink(URL_SITE.PAGENAME.'/'.$keywords.'/', $pagination['pages'], $pagination['currentsort'], 'relevance', $pagination['currentperpage']);
		redirect($redirectto);
	}
	
	$results = array();
	foreach ($ra as $item)
	{	
		if ($item['parent'])
		{
			$item['optionsummary'] = getParentProductOptionSummary($item['id']);
			$item['price'] = $item['optionsummary']['minprice'];
		}

	
		$item['variationcount']	= getProductVariationCount($item['id']);
		$item['manufacturer']	= getManufacturer($item['manufacturer_id']);
		$item['image']			= $db->get('productimages',$item['id'],'product_id', 'sortorder');
		$item['priceformatted']	= formatProductPrice($item['price']);
		$item['reviewcount'] = $db->value('productreviews', 'COUNT(*)', $item['id'], 'product_id');
		$item['offer'] = getProductOffer($item['id']);
		$results[] = $item;
	}
	
	$metanoindex = true;
	
	if ($pagination['currentpage'] > 1)
	{
		$page_title = $searchpage['metatitle'] . ': '.$keywords . ' - Page 2';
		$page_description = 'Page 2 - '.$searchpage['metadescription'].' - '.$keywords;
	}
	else
	{
		$page_title = $searchpage['metatitle'] . ': '.$keywords;
		$page_description = $searchpage['metadescription'].' - '.$keywords;
	}	
	
	$page_keywords = str_replace(' ',',',$keywords).','.$searchpage['metakeywords'];
	
	$keywords = stripslashes($keywords);
	$pagecontents = getFileContents(pickTemplateFile('search.php'));
	require(pickTemplateFile('layout_default.php'));