<?php
  
  class AmazonUSA
  {
      public $locale = "USA";
      public $version = "2011-08-01";
      public $url = "http://www.amazon.com/";
      public $ecsurl = "http://webservices.amazon.com/";
      public $requesturl = "GET\nwebservices.amazon.com\n/onca/xml\n";
      public $cartexpires = 7776000;
      public $tag = "";
      public $key = "";
      public $secretkey = "";
      public $supersavershippinglimit = 2500;
      public $config = array();
      public $cart = false;
      public $lastrequesterror = "";
      public $amazonmerchantid = "ATVPDKIKX0DER";
      public $marketplacedomain = "www.endless.com";
      
      public function AmazonUSA($tag, $key, $secretkey, &$db)
      {
          global $amazonconfigUSA;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigUSA;
          $this->db =& $db;
      }
      
      public function getTimestamp()
      {
          $timestamp = gmdate("Y-m-d\\TH:i:s\\Z");
          return $timestamp;
      }
      
      public function getSignature($parameterstring)
      {
          $signature = base64_encode(hash_hmac("sha256", $this->requesturl . $parameterstring, $this->secretkey, true));
          $signature = urlencode($signature);
          return $signature;
      }
      
      public function request($parameters, $urlonly = false)
      {
          $this->lastrequesterror       = "";
          $url                          = $this->ecsurl . "onca/xml";
          $parameters['Service']        = "AWSECommerceService";
          $parameters['AWSAccessKeyId'] = $this->key;
          $parameters['AssociateTag']   = $this->tag;
          $parameters['Version']        = $this->version;
          $parameters['Timestamp']      = $this->getTimestamp();
          ksort($parameters);
          $parameterstring = "";
          foreach ($parameters as $key => $item) {
              if ($key != "HMAC") {
                  $parameterstring .= "&" . $key . "=" . urlencode($item);
              } else {
                  $parameterstring .= "&" . $key . "=" . $item;
              }
          }
          $parameterstring = substr($parameterstring, 1);
          $signature       = $this->getSignature($parameterstring);
          $url .= "?" . $parameterstring . "&Signature=" . $signature;
          if ($urlonly) {
              return $url;
          }
          $dorequest = true;
          $i         = 0;
          while ($dorequest) {
              if (0 < $i) {
                  sleep(rand(1, 3));
              }
              ++$i;
              $xml = new XMLReader();
              if (defined("XML_FORCE_OPEN")) {
                  $xmlresult = @$xml->open($url);
              } else {
                  $xmlresult = $xml->xml(sendRemoteRequest($url));
              }
              if (!$xmlresult) {
                  $errors           = array();
                  $error            = array();
                  $error['code']    = "communicationserror";
                  $error['message'] = "The XML request failed, indicating a communications error";
                  $errors[]         = $error;
                  $this->processErrors($url, $errors, $i, $parameters);
                  if ($i < 4) {
                      continue;
                  }
                  break;
              }
              $data['url']       = $url;
              $data['requested'] = date("Y-m-d H:i:s");
              $this->db->insert("amazonrequests", $data);
              while (@$xml->read()) {
                  if (($xml->localName == "Errors" || $xml->localName == "ItemLookupErrorResponse") && $xml->nodeType == XMLReader::ELEMENT) {
                      $errors         = array();
                      $error          = array();
                      $founderrors    = true;
                      $finishederrors = false;
                      while (!$finishederrors) {
                          if ($xml->localName == "Code" && $xml->nodeType == XMLReader::ELEMENT) {
                              $xml->read();
                              $error['code'] = $xml->value;
                          }
                          if ($xml->localName == "Message" && $xml->nodeType == XMLReader::ELEMENT) {
                              $xml->read();
                              $error['message'] = $xml->value;
                          }
                          if ($xml->localName == "Error" && $xml->nodeType == XMLReader::END_ELEMENT) {
                              $errors[] = $error;
                              $error    = array();
                          }
                          if (($xml->localName == "Errors" || $xml->localName == "ItemLookupErrorResponse") && $xml->nodeType == XMLReader::END_ELEMENT) {
                              $finishederrors = true;
                          }
                          $xml->read();
                      }
                      $this->processErrors($url, $errors, $i, $parameters);
                      break;
                  }
                  if ($xml->localName == "Request" && $xml->nodeType == XMLReader::END_ELEMENT) {
                      break;
                  }
              }
              if (!$founderrors) {
                  return $xml;
              }
              $dorequest = $i < 3 ? true : false;
              if ($this->lastrequesterror == "accountlimitexceeded") {
                  $dorequest = false;
              }
              if ($this->lastrequesterror == "asinexpired") {
                  $dorequest = false;
              }
              if ($this->lastrequesterror == "nosearchresults") {
                  $dorequest = false;
              }
              $xml->close();
          }
          return false;
      }
      
      public function processErrors($url, $errors, $attempt, $parameters)
      {
          $errortype = "";
          foreach ($errors as $error) {
              $url = $this->db->cleanValue($url);
              if ($error['code'] == "AccountLimitExceeded") {
                  $this->lastrequesterror = "accountlimitexceeded";
                  $data                   = array();
                  $data['title']          = "Account Limit Exceeded";
                  $data['message']        = "Your Amazon API account request limit has been exceeded. Amazon grants a minimum of 2000 requests per hour, across all your sites.";
                  $data['item']           = "";
                  $data['identifier']     = "";
                  $data['created']        = "NOW()";
                  $this->db->insert("messages", $data);
              } else if ($error['code'] == "AWS.InvalidParameterValue" && $parameters['Operation'] == "ItemLookup" || $error['code'] = "AWS.ECommerceService.ItemNotAccessible") {
                  $this->lastrequesterror = "asinexpired";
                  $asin                   = $parameters['ItemId'];
                  $data                   = array();
                  $data['title']          = "Product Expired?";
                  $data['message']        = "The product with asin " . $asin . " was not found on the Amazon API. It may have expired, or this may be a temporary problem on Amazons API.";
                  $data['item']           = "product";
                  $data['identifier']     = "asin|{$asin}";
                  $data['created']        = "NOW()";
                  $this->db->insert("messages", $data);
              } else if ($error['code'] == "AWS.ECommerceService.NoExactMatches") {
                  $this->lastrequesterror = "nosearchresults";
              } else if ($error['code'] == "communicationserror") {
                  $this->lastrequesterror = "communications";
              } else {
                  $this->lastrequesterror = $error['code'];
              }
              $error['message'] = $this->db->cleanValue($error['message']);
              $sql              = "INSERT INTO amazonerrors (url, errorcode, errors, created, attempt) SELECT '{$url}', '" . $error['code'] . "', '" . $error['message'] . "', NOW(), {$attempt}";
              $this->db->query($sql);
          }
      }
      
      public function addToCart($offerlistingid, $asin, $quantity = 1)
      {
          if (!$this->cart) {
              $this->createCart($offerlistingid, $quantity);
          } else if (isset($this->cart['items'][$asin])) {
              $this->changeCartQuantity($asin, $quantity + $this->cart['items'][$asin]['quantity']);
          } else {
              $parameters['Operation']             = "CartAdd";
              $parameters['CartId']                = $this->cart['cartid'];
              $parameters['HMAC']                  = $this->cart['hmac'];
              $parameters['Item.1.OfferListingId'] = $offerlistingid;
              $parameters['Item.1.Quantity']       = $quantity;
              $xml                                 = $this->request($parameters);
              $cart                                = XMLObjectToArray($xml);
              $cart                                = simplifyXMLArray($cart);
              $this->getCart($cart);
          }
      }
      
      public function setCartCookies()
      {
          setcookie("CartId", $this->cart['cartid'], time() + $this->cartexpires + 60 * 60 * 24, "/");
          setcookie("HMAC", $this->cart['hmac'], time() + $this->cartexpires + 60 * 60 * 24, "/");
      }
      
      public function expireCartCookies()
      {
          setcookie("CartId", "", time() - 3600, "/");
          setcookie("HMAC", "", time() - 3600, "/");
      }
      
      public function createCart($offerlistingid, $quantity)
      {
          $parameters['Operation']             = "CartCreate";
          $parameters['Item.1.OfferListingId'] = $offerlistingid;
          $parameters['Item.1.Quantity']       = $quantity;
          $xml                                 = $this->request($parameters);
          $cart                                = XMLObjectToArray($xml);
          $cart                                = simplifyXMLArray($cart);
          $this->cart['cartid']                = $cart['CartId'];
          $this->cart['hmac']                  = $cart['URLEncodedHMAC'];
          $this->setCartCookies();
          $this->getCart($cart);
      }
      
      public function getCart($cart = false)
      {
          if (!$cart) {
              if (!isset($_COOKIE['HMAC']) || !isset($_COOKIE['CartId'])) {
                  $this->cart = false;
              } else {
                  $this->cart['hmac']      = $_COOKIE['HMAC'];
                  $this->cart['cartid']    = $_COOKIE['CartId'];
                  $parameters['Operation'] = "CartGet";
                  $parameters['CartId']    = $this->cart['cartid'];
                  $parameters['HMAC']      = $this->cart['hmac'];
                  $xml                     = $this->request($parameters);
                  if (!$xml) {
                      $this->expireCartCookies();
                      $this->cart = false;
                  } else {
                      $cart = XMLObjectToArray($xml);
                      $cart = simplifyXMLArray($cart);
                      $this->setCartCookies();
                  }
              }
          }
          if (!is_array($cart['CartItems']['CartItem']) || !isset($cart['PurchaseURL'])) {
              $this->expireCartCookies();
              $this->cart = false;
          } else {
              $this->cart['subtotal']    = $cart['SubTotal']['Amount'];
              $this->cart['purchaseurl'] = $cart['PurchaseURL'];
              if (isset($cart['CartItems']['CartItem']['CartItemId'])) {
                  $temp                            = $cart['CartItems']['CartItem'];
                  $cart['CartItems']['CartItem']   = array();
                  $cart['CartItems']['CartItem'][] = $temp;
              }
              $totalquantity       = 0;
              $this->cart['items'] = array();
              if (is_array($cart['CartItems']['CartItem'])) {
                  foreach ($cart['CartItems']['CartItem'] as $cartitem) {
                      $tempitem['cartitemid']                 = $cartitem['CartItemId'];
                      $tempitem['asin']                       = $cartitem['ASIN'];
                      $tempitem['merchantid']                 = $cartitem['MerchantId'];
                      $tempitem['sellerid']                   = $cartitem['SellerId'];
                      $tempitem['sellername']                 = $cartitem['SellerNickname'];
                      $tempitem['quantity']                   = $cartitem['Quantity'];
                      $tempitem['price']                      = $cartitem['Price']['Amount'];
                      $this->cart['items'][$tempitem['asin']] = $tempitem;
                      $totalquantity += $tempitem['quantity'];
                  }
              }
              $this->cart['totalquantity'] = $totalquantity;
          }
      }
      
      public function removeFromCart($asin)
      {
          return $this->changeCartQuantity($asin, 0);
      }
      
      public function changeCartQuantity($asin, $qty)
      {
          if (!isset($this->cart['items'][$asin])) {
              setMessage("error", "Item not found in your cart.");
              return false;
          }
          $oldquantity                     = $this->cart['items'][$asin]['quantity'];
          $parameters['Operation']         = "CartModify";
          $parameters['CartId']            = $this->cart['cartid'];
          $parameters['HMAC']              = $this->cart['hmac'];
          $parameters['Item.1.CartItemId'] = $this->cart['items'][$asin]['cartitemid'];
          $parameters['Item.1.Quantity']   = $qty;
          if ($parameters['Item.1.Quantity'] < 0) {
              $parameters['Item.1.Quantity'] = 0;
          }
          $xml  = $this->request($parameters);
          $cart = XMLObjectToArray($xml);
          $cart = simplifyXMLArray($cart);
          $this->getCart($cart);
          if ($this->cart['items'][$asin]['quantity'] < $qty) {
              setMessage("error", "There is currently only " . $this->cart['items'][$asin]['quantity'] . " available of this item.");
              return false;
          }
          return true;
      }
      
      public function getProductImages($asin)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Images";
          $parameters['ItemId']        = $asin;
          $xml                         = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          $imagesets = array();
          while ($xml->read()) {
              if ($xml->localName == "ImageSets" && $xml->nodeType == XMLReader::ELEMENT) {
                  $imagesets = XMLObjectToArray($xml);
              }
          }
          $returnimages = array();
          foreach ($imagesets as $imageset) {
              if ($imageset['tag'] != "ImageSet") {
                  continue;
              }
              $imageurl = $imageset['value'][0]['value'][0]['value'];
              $imageurl = explode("._SL30_.", $imageurl);
              $imageurl = $imageurl[0] . "{PARAMETERS}" . $imageurl[1];
              $imageurl = str_replace("_SL30", "_SLnn", $imageurl);
              if (!in_array($imageurl, $returnimages)) {
                  $returnimages[] = $imageurl;
              }
          }
          return $returnimages;
      }
      
      public function getProductPromotions($asin)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "PromotionDetails,Offers";
          $parameters['ItemId']        = $asin;
          $xml                         = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          $images = array();
          while ($xml->read()) {
              if ($xml->localName == "ImageSets" && $xml->nodeType == XMLReader::ELEMENT) {
                  $imagesets = XMLObjectToArray($xml);
              }
          }
      }
      
      public function getProductOffers($asin, $merchantid = "All", $marketplaceitem = false)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Offers";
          $parameters['ItemId']        = $asin;
          $parameters['MerchantId']    = $merchantid;
          if ($marketplaceitem) {
              $parameters['MarketplaceDomain'] = $this->marketplacedomain;
          }
          $offers = array();
          $xml    = $this->request($parameters);
          if (!$xml) {
              return $offers;
          }
          while ($xml->read()) {
              if ($xml->localName == "Offer" && $xml->nodeType == XMLReader::ELEMENT) {
                  $offer      = XMLObjectToArray($xml);
                  $attributes = $offer[0]['value'];
                  $listing    = $offer[1]['value'];
                  $tempoffer  = array();
                  foreach ($attributes as $item) {
                      $tempoffer['attributes'][strtolower($item['tag'])] = $item['value'];
                      if ($item['tag'] == "Condition") {
                          $tempoffer['condition'] = $item['value'];
                      }
                  }
                  foreach ($listing as $item) {
                      if ($item['tag'] == "OfferListingId") {
                          $tempoffer['offerlistingid'] = $item['value'];
                      }
                      if ($item['tag'] == "PercentageSaved") {
                          $tempoffer['percentagesaved'] = $item['value'];
                      }
                      if ($item['tag'] == "Availability") {
                          $tempoffer['availabilitytext'] = $item['value'];
                      }
                      if ($item['tag'] == "Quantity") {
                          $tempoffer['quantity'] = $item['value'];
                      }
                      if ($item['tag'] == "IsEligibleForSuperSaverShipping") {
                          $tempoffer['supersavershipping'] = $item['value'];
                      }
                      if ($item['tag'] == "Price") {
                          foreach ($item['value'] as $pricevalue) {
                              if ($pricevalue['tag'] == "Amount") {
                                  $tempoffer['price'] = $pricevalue['value'];
                              }
                          }
                      }
                      if ($item['tag'] == "SalePrice") {
                          foreach ($item['value'] as $pricevalue) {
                              if ($pricevalue['tag'] == "Amount") {
                                  $tempoffer['price'] = $pricevalue['value'];
                              }
                          }
                      }
                      if ($item['tag'] == "AmountSaved") {
                          foreach ($item['value'] as $savedvalue) {
                              if ($savedvalue['tag'] == "Amount") {
                                  $tempoffer['amountsaved'] = $savedvalue['value'];
                              }
                          }
                      }
                      if ($item['tag'] == "AvailabilityAttributes") {
                          foreach ($item['value'] as $attributesvalue) {
                              if ($attributesvalue['tag'] == "AvailabilityType") {
                                  $tempoffer['availabilitytype'] = $attributesvalue['value'];
                              }
                              if ($attributesvalue['tag'] == "MinimumHours") {
                                  $tempoffer['availabilitymin'] = $attributesvalue['value'];
                              }
                              if ($attributesvalue['tag'] == "MaximumHours") {
                                  $tempoffer['availabilitymax'] = $attributesvalue['value'];
                              }
                              if ($attributesvalue['tag'] == "IsPreorder") {
                                  $tempoffer['ispreorder'] = $attributesvalue['value'];
                              }
                          }
                      }
                  }
                  if (!isset($tempoffer['ispreorder'])) {
                      $tempoffer['ispreorder'] = 0;
                  }
                  if (!isset($tempoffer['quantity'])) {
                      $tempoffer['quantity'] = 0;
                  }
                  $offers[] = $tempoffer;
              }
          }
          $xml->close();
          return $offers;
      }
      
      public function getProductVariationOffers($asin, $callback)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Variations";
          $parameters['ItemId']        = $asin;
          $parameters['MerchantId']    = "All";
          $xml                         = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          $itemcount = 0;
          while ($xml->read()) {
              if ($xml->localName == "Item" && $xml->nodeType == XMLReader::ELEMENT) {
                  ++$itemcount;
                  if (1 < $itemcount) {
                      $product     = XMLObjectToArray($xml);
                      $productasin = $product[0]['value'];
                      $offers      = array();
                      foreach ($product as $item) {
                          if ($item['tag'] == "Offers") {
                              $offers = $item['value'];
                          }
                      }
                      foreach ($offers as $tempoffer) {
                          if (isset($tempoffer['tag'])) {
                              $t           = $tempoffer;
                              $tempoffer   = array();
                              $tempoffer[] = $t;
                          }
                          $tempoffer                   = simplifyXMLArray($tempoffer);
                          $tempoffer                   = $tempoffer['Offer'];
                          $offer                       = array();
                          $offer['asin']               = $productasin;
                          $offer['condition']          = $tempoffer['OfferAttributes']['Condition'];
                          $offer['offerlistingid']     = $tempoffer['OfferListing']['OfferListingId'];
                          $offer['price']              = isset($tempoffer['OfferListing']['SalePrice']['Amount']) ? $tempoffer['OfferListing']['SalePrice']['Amount'] : $tempoffer['OfferListing']['Price']['Amount'];
                          $offer['amountsaved']        = isset($tempoffer['OfferListing']['AmountSaved']['Amount']) ? $tempoffer['OfferListing']['AmountSaved']['Amount'] : 0;
                          $offer['percentagesaved']    = isset($tempoffer['OfferListing']['PercentageSaved']) ? $tempoffer['OfferListing']['PercentageSaved'] : 0;
                          $offer['availabilitytext']   = isset($tempoffer['OfferListing']['Availablility']) ? $tempoffer['OfferListing']['Availablility'] : "";
                          $offer['availabilitytype']   = isset($tempoffer['OfferListing']['AvailablilityAttributes']['AvailabilityType']) ? $tempoffer['OfferListing']['AvailablilityAttributes']['AvailabilityType'] : "";
                          $offer['availabilitymin']    = isset($tempoffer['OfferListing']['AvailablilityAttributes']['MinimumHours']) ? $tempoffer['OfferListing']['AvailablilityAttributes']['MinimumHours'] : "";
                          $offer['availabilitymax']    = isset($tempoffer['OfferListing']['AvailablilityAttributes']['MaximumHours']) ? $tempoffer['OfferListing']['AvailablilityAttributes']['MaximumHours'] : "";
                          $offer['quantity']           = isset($tempoffer['OfferListing']['Quantity']) ? $tempoffer['OfferListing']['Quantity'] : 0 - 1;
                          $offer['supersavershipping'] = $tempoffer['OfferListing']['IsEligibleForSuperSaverShipping'];
                          if (isset($tempoffer['Merchant'], $tempoffer['Merchant']['MerchantId'])) {
                              $offer['merchantid']                = $tempoffer['Merchant']['MerchantId'];
                              $offer['merchant']['merchantid']    = $tempoffer['Merchant']['MerchantId'];
                              $offer['merchant']['averagerating'] = $tempoffer['Merchant']['AverageFeedbackRating'];
                              $offer['merchant']['totalratings']  = $tempoffer['Merchant']['TotalFeedback'];
                          }
                          $callback($offer);
                      }
                  }
              }
          }
          return true;
      }
      
      public function getProductReviews($asin)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Reviews";
          $parameters['ItemId']        = $asin;
          $returnreviews               = array();
          $reviewinfo                  = array();
          $totalpages                  = 2;
          $i                           = 1;
          while ($i <= $totalpages) {
              $parameters['ReviewPage'] = $i;
              $xml                      = $this->request($parameters);
              if (!$xml) {
                  return $returnreviews;
              }
              while ($xml->read()) {
                  if ($xml->localName == "TotalReviewPages" && $xml->nodeType == XMLReader::ELEMENT) {
                  }
                  if ($xml->localName == "AverageRating" && $xml->nodeType == XMLReader::ELEMENT) {
                      $reviewinfo['averagescore'] = XMLObjectToArray($xml);
                  }
                  if ($xml->localName == "TotalReviews" && $xml->nodeType == XMLReader::ELEMENT) {
                      $reviewinfo['totalreviews'] = XMLObjectToArray($xml);
                  }
                  if ($xml->localName == "IFrameURL" && $xml->nodeType == XMLReader::ELEMENT) {
                      $reviewinfo['iframeurl'] = XMLObjectToArray($xml);
                  }
                  if ($xml->localName == "Review" && $xml->nodeType == XMLReader::ELEMENT) {
                      $review                   = XMLObjectToArray($xml);
                      $review                   = simplifyXMLArray($review);
                      $temp                     = array();
                      $temp['rating']           = $review['Rating'];
                      $temp['helpfulvotes']     = $review['HelpfulVotes'];
                      $temp['totalvotes']       = $review['TotalVotes'];
                      $temp['customerid']       = isset($review['Reviewer']['CustomerId']) ? $review['Reviewer']['CustomerId'] : "";
                      $temp['reviewername']     = isset($review['Reviewer']['Name']) ? $review['Reviewer']['Name'] : "";
                      $temp['reviewernickname'] = isset($review['Reviewer']['Nickname']) ? $review['Reviewer']['Nickname'] : "";
                      $temp['reviewerlocation'] = isset($review['Reviewer']['Location']) ? $review['Reviewer']['Location'] : "";
                      $temp['date']             = $review['Date'];
                      $temp['title']            = isset($review['Summary']) ? $review['Summary'] : "";
                      $temp['review']           = $review['Content'];
                      $returnreviews[]          = $temp;
                  }
              }
              ++$i;
          }
          $return               = array();
          $return['reviewinfo'] = $reviewinfo;
          $return['reviews']    = $returnreviews;
          return $return;
      }
      
      public function getProductsByBrowseNode($browsenode, $searchindex, $lastpage = 1, $merchantid = "", $minprice = "", $maxprice = "")
      {
          $page = $lastpage + 1;
          if (10 < $page) {
              $page = 1;
          }
          $parameters['Operation']     = "ItemSearch";
          $parameters['ResponseGroup'] = "Medium,VariationSummary";
          $parameters['BrowseNode']    = urlencode($browsenode);
          $parameters['SearchIndex']   = $searchindex;
          if ($searchindex == "Marketplace") {
              $parameters['MarketplaceDomain'] = $this->marketplacedomain;
          }
          $parameters['ItemPage'] = $page;
          if (trim($merchantid) != "") {
              $parameters['MerchantId'] = $merchantid;
          }
          if ($minprice != 0) {
              $parameters['MinimumPrice'] = str_replace(".", "", $minprice);
          }
          if ($maxprice != 0) {
              $parameters['MaximumPrice'] = str_replace(".", "", $maxprice);
          }
          $xml                = $this->request($parameters);
          $return['page']     = $page;
          $return['products'] = array();
          $return['page']     = $page;
          $return['products'] = array();
          if (!$xml || empty($xml)) {
              return $return;
          }
          while ($xml->read()) {
              if ($xml->localName == "TotalPages" && $xml->nodeType == XMLReader::ELEMENT) {
                  $totalpages = XMLObjectToArray($xml);
              }
              if ($xml->localName == "Item" && $xml->nodeType == XMLReader::ELEMENT) {
                  $item                               = XMLObjectToArray($xml);
                  $item                               = simplifyXMLArray($item);
                  $product                            = array();
                  $product['core']['asin']            = $item['ASIN'];
                  $product['core']['url']             = $item['DetailPageURL'];
                  $product['core']['salesrank']       = isset($item['SalesRank']) ? $item['SalesRank'] : 0;
                  $product['core']['parent']          = isset($item['VariationSummary']) ? 1 : 0;
                  $product['core']['title']           = formatText($item['ItemAttributes']['Title']);
                  $product['core']['description']     = isset($item['EditorialReviews']['EditorialReview']['Content']) ? htmlentities($item['EditorialReviews']['EditorialReview']['Content']) : "";
                  $product['core']['marketplaceitem'] = $searchindex == "Marketplace" ? 1 : 0;
                  if (isset($item['ItemAttributes']['Feature'])) {
                      if (is_array($item['ItemAttributes']['Feature'])) {
                          $product['core']['keyfeatures'] = implode("\n", $item['ItemAttributes']['Feature']);
                      } else {
                          $product['core']['keyfeatures'] = $item['ItemAttributes']['Feature'];
                      }
                  } else {
                      $product['core']['keyfeatures'] = "";
                  }
                  if (!isset($item['ImageSets']['ImageSet'])) {
                      $product['images'] = false;
                  } else {
                      if (isset($item['ImageSets']['ImageSet']['SwatchImage'])) {
                          $singleimage                     = $item['ImageSets']['ImageSet'];
                          $item['ImageSets']['ImageSet']   = array();
                          $item['ImageSets']['ImageSet'][] = $singleimage;
                      }
                      foreach ($item['ImageSets']['ImageSet'] as $image) {
                          $imageurl            = $image['SwatchImage']['URL'];
                          $imageurl            = explode("._SL30_.", $imageurl);
                          $imageurl            = $imageurl[0] . "{PARAMETERS}" . $imageurl[1];
                          $imageurl            = str_replace("_SL30", "_SLnn", $imageurl);
                          $product['images'][] = $imageurl;
                      }
                  }
                  $product['attributes']       = $this->parseAttributes($item['ItemAttributes']);
                  $product['editorialreviews'] = isset($item['EditorialReviews']) ? $item['EditorialReviews'] : false;
                  $return['products'][]        = $product;
              }
          }
          if ($totalpages == $return['page']) {
              $return['page'] = 0;
          }
          return $return;
      }
      
      public function getProductsBySearch($searchindex, $keywords, $lastpage = 1, $merchantid = "", $minprice = "", $maxprice = "")
      {
          $page = $lastpage + 1;
          if (10 < $page) {
              $page = 1;
          }
          $parameters['Operation']     = "ItemSearch";
          $parameters['ResponseGroup'] = "Medium,VariationSummary";
          $parameters['Keywords']      = urlencode($keywords);
          $parameters['SearchIndex']   = $searchindex;
          if ($searchindex == "Marketplace") {
              $parameters['MarketplaceDomain'] = $this->marketplacedomain;
          }
          $parameters['ItemPage'] = $page;
          if (trim($merchantid) != "") {
              $parameters['MerchantId'] = $merchantid;
          }
          if ($minprice != 0) {
              $parameters['MinimumPrice'] = str_replace(".", "", $minprice);
          }
          if ($maxprice != 0) {
              $parameters['MaximumPrice'] = str_replace(".", "", $maxprice);
          }
          $xml                = $this->request($parameters);
          $return['page']     = $page;
          $return['products'] = array();
          if (!$xml || empty($xml)) {
              return $return;
          }
          while ($xml->read()) {
              if ($xml->localName == "TotalPages" && $xml->nodeType == XMLReader::ELEMENT) {
                  $totalpages = XMLObjectToArray($xml);
                  if ($totalpages == $return['page']) {
                      $return['page'] = 1;
                  }
              }
              if ($xml->localName == "Item" && $xml->nodeType == XMLReader::ELEMENT) {
                  $item                               = XMLObjectToArray($xml);
                  $item                               = simplifyXMLArray($item);
                  $product                            = array();
                  $product['core']['asin']            = is_array($item['ASIN']) ? array_pop($item['ASIN']) : $item['ASIN'];
                  $product['core']['url']             = $item['DetailPageURL'];
                  $product['core']['salesrank']       = isset($item['SalesRank']) ? $item['SalesRank'] : 0;
                  $product['core']['parent']          = isset($item['VariationSummary']) ? 1 : 0;
                  $product['core']['title']           = formatText($item['ItemAttributes']['Title']);
                  $product['core']['description']     = isset($item['EditorialReviews']['EditorialReview']['Content']) ? htmlentities($item['EditorialReviews']['EditorialReview']['Content']) : "";
                  $product['core']['keyfeatures']     = is_array($item['ItemAttributes']['Feature']) ? implode("\n", $item['ItemAttributes']['Feature']) : "";
                  $product['core']['marketplaceitem'] = $searchindex == "Marketplace" ? 1 : 0;
                  if (!isset($item['ImageSets']['ImageSet'])) {
                      $product['images'] = false;
                  } else {
                      if (isset($item['ImageSets']['ImageSet']['SwatchImage'])) {
                          $singleimage                     = $item['ImageSets']['ImageSet'];
                          $item['ImageSets']['ImageSet']   = array();
                          $item['ImageSets']['ImageSet'][] = $singleimage;
                      }
                      foreach ($item['ImageSets']['ImageSet'] as $image) {
                          $imageurl            = $image['SwatchImage']['URL'];
                          $imageurl            = explode("._SL30_.", $imageurl);
                          $imageurl            = $imageurl[0] . "{PARAMETERS}" . $imageurl[1];
                          $imageurl            = str_replace("_SL30", "_SLnn", $imageurl);
                          $product['images'][] = $imageurl;
                      }
                  }
                  $product['attributes']       = $this->parseAttributes($item['ItemAttributes']);
                  $product['editorialreviews'] = isset($item['EditorialReviews']) ? $item['EditorialReviews'] : false;
                  $return['products'][]        = $product;
              }
          }
          return $return;
      }
      
      public function getProductBasic($asin)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Small,VariationSummary,SalesRank";
          $parameters['ItemId']        = $asin;
          $parameters['MerchantId']    = "All";
          $xml                         = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          $product = array();
          while ($xml->read()) {
              if ($xml->localName == "Item" && $xml->nodeType == XMLReader::ELEMENT) {
                  $item                 = XMLObjectToArray($xml);
                  $item                 = simplifyXMLArray($item);
                  $product['asin']      = $item['ASIN'];
                  $product['url']       = $item['DetailPageURL'];
                  $product['salesrank'] = isset($item['SalesRank']) ? $item['SalesRank'] : 0;
                  $product['parent']    = isset($item['VariationSummary']) ? 1 : 0;
              }
          }
          return $product;
      }
      
      public function parseAttributes($attributes)
      {
          $return = array();
          foreach ($attributes as $key => $item) {
              if (is_array($item)) {
                  foreach ($item as $subkey => $subitem) {
                      $newkey   = $key . $subkey;
                      $newvalue = is_array($subitem) ? $subitem['value'] : $subitem;
                      if (isset($subitem['attributes']) && is_array($subitem['attributes'])) {
                          foreach ($subitem['attributes'] as $a_key => $a_item) {
                              $newvalue .= " " . $a_key . " " . $a_item;
                          }
                      }
                      $return[$newkey] = $newvalue;
                  }
              } else {
                  $return[$key] = $item;
              }
          }
          return $return;
      }
      
      public function getProductFull($asin)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Medium,VariationSummary";
          $parameters['ItemId']        = $asin;
          $parameters['MerchantId']    = "All";
          $xml                         = $this->request($parameters);
          if (!$xml) {
              if ($this->lastrequesterror == "asinexpired") {
                  $this->db->query("UPDATE products SET expired = 1 WHERE asin = '{$asin}'");
              }
              return false;
          }
          $product = array();
          while ($xml->read()) {
              if ($xml->localName == "Item" && $xml->nodeType == XMLReader::ELEMENT) {
                  $item                         = XMLObjectToArray($xml);
                  $item                         = simplifyXMLArray($item);
                  $product['core']['asin']      = $item['ASIN'];
                  $product['core']['url']       = $item['DetailPageURL'];
                  $product['core']['salesrank'] = isset($item['SalesRank']) ? $item['SalesRank'] : 0;
                  $product['core']['parent']    = isset($item['VariationSummary']) ? 1 : 0;
                  $product['core']['title']     = $item['ItemAttributes']['Title'];
                  if (isset($item['EditorialReviews']['EditorialReview']['Content'])) {
                      $content = htmlentities(( boolean ) $item['EditorialReviews']['EditorialReview']['Content']);
                  } else if (isset($item['EditorialReviews']['EditorialReview'][0]['Content'])) {
                      $content = "";
                      foreach ($item['EditorialReviews']['EditorialReview'] as $edcontent) {
                          $content .= htmlentities(( boolean ) $edcontent['Content']) . "<br /><br />";
                      }
                  } else {
                      $content == "";
                  }
                  $product['core']['description'] = $content;
                  if (isset($item['ItemAttributes']['Feature'])) {
                      if (is_array($item['ItemAttributes']['Feature'])) {
                          $product['core']['keyfeatures'] = implode("\n", $item['ItemAttributes']['Feature']);
                      } else {
                          $product['core']['keyfeatures'] = $item['ItemAttributes']['Feature'];
                      }
                  }
                  if (!isset($item['ImageSets']['ImageSet'])) {
                      $product['images'] = false;
                  } else {
                      if (isset($item['ImageSets']['ImageSet']['SwatchImage'])) {
                          $images[] = $item['ImageSets']['ImageSet'];
                      } else {
                          $images = $item['ImageSets']['ImageSet'];
                      }
                      foreach ($images as $image) {
                          $imageurl            = $image['SwatchImage']['URL'];
                          $imageurl            = explode("._SL30_.", $imageurl);
                          $imageurl            = $imageurl[0] . "{PARAMETERS}" . $imageurl[1];
                          $imageurl            = str_replace("_SL30", "_SLnn", $imageurl);
                          $product['images'][] = $imageurl;
                      }
                  }
                  if (!is_array($item['ItemAttributes'])) {
                  }
                  $product['attributes'] = $this->parseAttributes($item['ItemAttributes']);
                  if (isset($product['attributes']['Feature'])) {
                      if (is_array($product['attributes']['Feature'])) {
                          $product['core']['keyfeatures'] = implode("\n", $product['attributes']['Feature']);
                      } else {
                          $product['core']['keyfeatures'] = $product['attributes']['Feature'];
                      }
                  }
                  break;
              }
          }
          return $product;
      }
      
      public function getProductVariations($asin, $page = 1)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "Medium,VariationMatrix";
          $parameters['ItemId']        = $asin;
          $parameters['MerchantId']    = "All";
          $parameters['Condition']     = "All";
          $xml                         = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          while ($xml->read()) {
              if ($xml->localName == "Item" && $xml->nodeType == XMLReader::ELEMENT) {
                  $item       = XMLObjectToArray($xml);
                  $item       = simplifyXMLArray($item);
                  $dimensions = $item['Variations']['VariationDimensions']['VariationDimension'];
                  $children   = $item['Variations']['Item'];
                  if (isset($children['ASIN'])) {
                      $temp       = $children;
                      $children   = array();
                      $children[] = $temp;
                  }
                  if (!$children) {
                      $children = array();
                  }
                  foreach ($children as $key => $value) {
                      if (!is_array($value['VariationAttributes'])) {
                          $children[$key]['VariationAttributes'] = array();
                          continue;
                      }
                      $attributes = array();
                      foreach ($children[$key]['VariationAttributes'] as $thiskey => $item) {
                          if (isset($item[0])) {
                              foreach ($item as $thisitem) {
                                  $attributes[$thisitem['Name']] = $thisitem['Value'];
                              }
                          } else {
                              $attributes[$item['Name']] = $item['Value'];
                          }
                      }
                      $children[$key]['ItemAttributes'] = $attributes;
                  }
                  $variations['items']      = $children;
                  $variations['dimensions'] = $dimensions;
                  break;
              }
          }
          return $variations;
      }
      
      public function getProductAttributes($asin)
      {
          $parameters['Operation']     = "ItemLookup";
          $parameters['ResponseGroup'] = "ItemAttributes";
          $parameters['ItemId']        = $asin;
          $xml                         = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          while ($xml->read()) {
              if ($xml->localName == "ItemAttributes" && $xml->nodeType == XMLReader::ELEMENT) {
                  $item = XMLObjectToArray($xml);
                  $item = simplifyXMLArray($item);
                  if (!is_array($item)) {
                      print_r($item);
                      print_r($parameters);
                  }
                  $attributes = $this->parseAttributes($item);
                  break;
              }
          }
          return $attributes;
      }
      
      public function testBrowseNode($browsenode)
      {
          $parameters['Operation']    = "BrowseNodeLookup";
          $parameters['BrowseNodeId'] = $browsenode;
          $xml                        = $this->request($parameters);
          if (!$xml) {
              return false;
          }
          while ($xml->read()) {
              if ($xml->localName == "Errors" && $xml->nodeType == XMLReader::ELEMENT) {
                  return false;
              }
          }
          return true;
      }
      
      public function createBrowseNodeLink($browsenode)
      {
          $link = $this->url . "b?ie=UTF8&node={$browsenode}&tag=" . $this->tag;
          return $link;
      }
      
      public function createSearchTermLink($searchterm)
      {
          $link = $this->url . "s?url=search-alias%3Daps&field-keywords=" . urlencode($searchterm) . "&tag=" . $this->tag;
          return $link;
      }
      
      public function createAddToCartLink($asin, $qty = 1)
      {
          $link = $this->url . "gp/aws/cart/add.html";
          $link .= "?AssociateTag=" . $this->tag;
          $link .= "&SubscriptionId=" . $this->key;
          $link .= "&ASIN.1=" . $asin;
          $link .= "&Quantity.1=" . $qty;
          return $link;
      }
      
  }
  
  class AmazonUK extends AmazonUSA
  {
      public function AmazonUK($tag, $key, $secretkey, $db)
      {
          global $amazonconfigUK;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigUK;
          $this->db =& $db;
      }
      
  }
  
  class AmazonDE extends AmazonUSA
  {
      public function AmazonDE($tag, $key, $secretkey, $db)
      {
          global $amazonconfigDE;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigDE;
          $this->db =& $db;
      }
      
  }
  
  class AmazonIT extends AmazonUSA
  {
      public function AmazonIT($tag, $key, $secretkey, $db)
      {
          global $amazonconfigIT;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigIT;
          $this->db =& $db;
      }
      
  }
  
  class AmazonJP extends AmazonUSA
  {
      public function AmazonJP($tag, $key, $secretkey, $db)
      {
          global $amazonconfigJP;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigJP;
          $this->db =& $db;
      }
      
  }
  
  class AmazonCA extends AmazonUSA
  {
      public function AmazonCA($tag, $key, $secretkey, $db)
      {
          global $amazonconfigCA;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigCA;
          $this->db =& $db;
      }
      
  }
  
  class AmazonFR extends AmazonUSA
  {
      public function AmazonFR($tag, $key, $secretkey, $db)
      {
          global $amazonconfigFR;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigFR;
          $this->db =& $db;
      }
      
  }
  
  class AmazonES extends AmazonUSA
  {
      public function AmazonES($tag, $key, $secretkey, $db)
      {
          global $amazonconfigES;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigFR;
          $this->db =& $db;
      }
      
  }
  
  class AmazonCN extends AmazonUSA
  {
      public function AmazonCN($tag, $key, $secretkey, $db)
      {
          global $amazonconfigCN;
          $this->tag       = $tag;
          $this->key       = $key;
          $this->secretkey = $secretkey;
          $this->config    = $amazonconfigFR;
          $this->db =& $db;
      }
      
  }
  
  function initialiseAmazon()
  {
      global $db;
      $locale = checkAmazonTagLocale(AMAZON_TAG);
      if ($locale == "UK") {
          $amazon = new AmazonUK(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "USA") {
          $amazon = new AmazonUSA(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "DE") {
          $amazon = new AmazonDE(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "IT") {
          $amazon = new AmazonIT(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "JP") {
          $amazon = new AmazonJP(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "FR") {
          $amazon = new AmazonFR(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "CA") {
          $amazon = new AmazonCA(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "ES") {
          $amazon = new AmazonES(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else if ($locale == "CN") {
          $amazon = new AmazonCN(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      } else {
          $amazon = new AmazonUSA(AMAZON_TAG, AMAZON_KEY, AMAZON_SECRET_KEY, $db);
      }
      checkLicense($amazon);
      return $amazon;
  }
  
  function checkAmazonTagLocale($tag)
  {
      if (STORE_LOCALE == "USA" || STORE_LOCALE == "UK" || STORE_LOCALE == "DE" || STORE_LOCALE == "IT" || STORE_LOCALE == "JP" || STORE_LOCALE == "FR" || STORE_LOCALE == "CA" || STORE_LOCALE == "CN" || STORE_LOCALE == "ES") {
          return STORE_LOCALE;
      }
      $tag    = explode("-", $tag);
      $number = $tag[sizeof($tag) - 1];
      if ($number == "21") {
          $locale = "UK";
      }
      if ($number == "20") {
          $locale = "USA";
      }
      global $db;
      $db->query("UPDATE parameters SET value = '{$locale}' WHERE parameter = 'STORE_LOCALE'");
      return $locale;
  }
  
  function checkLicense($amazon)
  {
  return true;
  /*
      $cachefilename = md5($_SERVER['HTTP_HOST'] . $amazon->tag);
      $callback      = true;
      if (file_exists(PATH_ROOT . "cache/" . $cachefilename)) {
          $cachedinfo = file_get_contents(PATH_ROOT . "cache/" . $cachefilename);
          $cachedinfo = unserialize(base64_decode($cachedinfo));
          $checksum   = hash("sha512", "+3j5=(*xxD" . $cachedinfo['expires']);
          if ($checksum == $cachedinfo['checksum'] && time() < $cachedinfo['expires']) {
              $callback = false;
              $details  = $cachedinfo;
          }
      }
      if ($callback) {
          $get['c']            = "callback";
          $get['url']          = $_SERVER['HTTP_HOST'];
          $get['parameterurl'] = $parameters['URL_SITE'];
          if (trim($get['url']) == "") {
              $get['url'] = $get['parameterurl'];
          }
          $get['serverip']       = $_SERVER['SERVER_ADDR'];
          $get['serverport']     = $_SERVER['SERVER_PORT'];
          $get['serversoftware'] = $_SERVER['SERVER_SOFTWARE'];
          $get['tag']            = $amazon->tag;
          $get['scriptfilename'] = $_SERVER['SCRIPT_FILENAME'];
          $get['fromscript']     = "amazon class";
          $url                   = "http://callback.freshstorebuilder.c0m/index.php?";
          foreach ($get as $key => $item) {
              $url .= "{$key}=" . urlencode($item) . "&";
          }
          if (function_exists("curl_init")) {
              $ch = curl_init($url);
              curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
              curl_setopt($ch, CURLOPT_TIMEOUT, 300);
              $response = curl_exec($ch);
              curl_close($ch);
          } else if (intval(get_cfg_var("allow_url_fopen")) && function_exists("file")) {
              if ($response = file($url)) {
                  $response = join("", $response);
              }
          } else if (intval(get_cfg_var("allow_url_fopen")) && function_exists("file_get_contents")) {
              $response = file_get_contents($url);
          } else {
              exit("cURL or allow_url_fopen needs to be enabled on this server.");
          }
          if (!$response) {
              $filecontents        = array();
              $filecontents['s']   = "c";
              $filecontents['tuk'] = "fa21-21";
              $filecontents['tus'] = "fa20-20";
              $filecontents['tde'] = "fa20-20";
              $filecontents['tit'] = "";
              $filecontents['tjp'] = "";
          } else {
              $filecontents = unserialize($response);
          }
          $filecontents['created']  = time();
          $filecontents['expires']  = time() + $filecontents['c'];
          $filecontents['checksum'] = hash("sha512", "+3j5=(*xxD" . $filecontents['expires']);
          $details                  = $filecontents;
          $filecontents             = base64_encode(serialize($filecontents));
          file_put_contents(PATH_ROOT . "cache/" . $cachefilename, $filecontents);
      }
      if ($details['s'] == "e") {
          echo "Invalid License";
          exit();
      } else if ($details['s'] == "c") {
          if ($amazon->locale == "UK") {
              $amazon->tag = $details['tuk'];
          }
          if ($amazon->locale == "USA") {
              $amazon->tag = $details['tus'];
          }
          if ($amazon->locale == "DE") {
              $amazon->tag = $details['tde'];
          }
          if ($amazon->locale == "IT") {
              $amazon->tag = $details['tit'];
          }
          if ($amazon->locale == "JP") {
              $amazon->tag = $details['jp'];
          }
      }
	  */
  }
  
  function isStoreLicensed()
  {
  return true;
  /*
      global $amazon;
      $cachefilename = md5($_SERVER['HTTP_HOST'] . $amazon->tag);
      if (file_exists(PATH_ROOT . "cache/" . $cachefilename)) {
          $cachedinfo = file_get_contents(PATH_ROOT . "cache/" . $cachefilename);
          $cachedinfo = unserialize(base64_decode($cachedinfo));
          $checksum   = hash("sha512", "+3j5=(*xxD" . $cachedinfo['expires']);
          if ($checksum == $cachedinfo['checksum'] && time() < $cachedinfo['expires']) {
              $details = $cachedinfo;
          }
      }
      if ($details['s'] == "v") {
          return true;
      }
      return false;
	  */
  }
  
?>