<?php
	
	# Sub Libraries
	require('categories_autoupdate.php');
	
	function validateCategory($category)
	{		
		if (!isset($category['title']) || $category['title'] == '' )
		{
			setMessage('error', 'Category title is required');
			return false;
		}
		
		if (strlen($category['metadescription']) > 180)
		{
			setMessage('information', 'Meta description should be no longer than 180 characters. (Currently ' . strlen($category['metadescription']) . ' characters)');
		}
		
		return true;
	}
	
	function generateCategoryFields($category, $category_id = 0)
	{
		if (!isset($category['pagename']) || $category['pagename'] == '')
		{
			$category['pagename'] = createPageName(array($category['title']), array('category_id' => $category_id));
		}
		else
		{
			$category['pagename'] = getUniquePageName($category['pagename'], array('category_id' => $category_id));
			$category['pagename'] = tidyPagename($category['pagename']);
		}
		
		if (!isset($category['metatitle']) || $category['metatitle'] == '')
			$category['metatitle'] = $category['title'];
			
		if (!isset($category['linktitle']) || $category['linktitle'] == '')
			$category['linktitle'] = $category['title'];
		
		if (!isset($category['metadescription']) || $category['metadescription'] == '')
		{
			if (!isset($category['description']) || $category['description'] == '')
			{
				$category['metadescription'] = $category['title'] . ' at ' . SITE_NAME.'.';
			}
			else
			{
				$category['metadescription'] = strip_tags($category['description']);
			}
			
			# Make sure it isn't too big
			if (strlen($category['metadescription']) > 180)
			{
				$category['metadescription'] = substr($category['metadescription'], 0, $size-3).'...';
			}
		}
		
		if (!isset($category['metakeywords']) || $category['metakeywords'] == '')
			$category['metakeywords'] = createKeywordString($category);
		
		return ($category);
	}

	function addCategory($category)
	{
		global $db;
		
		if (!validateCategory($category))
		{
			return false;
		}
		
		$category = generateCategoryFields($category);
		
		$category_id = $db->insert('categories', $category);
		return ($category_id);
	}
	
	function saveCategory($category_id, $category)
	{
		global $db;
		
		if (!validateCategory($category))
		{
			return false;
		}
		
		$category = generateCategoryFields($category, $category_id);
		
		# Check for a pagename change, insert into legacy
		$oldpagename = $db->value('categories', 'pagename', $category_id);
		if (isset($category['pagename']) && $oldpagename != $category['pagename'])
		{
			setPageRedirect($oldpagename, $category['pagename']);
		}
		
		$db->update('categories', $category_id, $category);
		return true;
	}
	
	function getCategory($category_id)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		$sql = "SELECT * FROM categories WHERE id = '{$category_id}'";
		$rs = $db->query($sql);
		$category = $rs->fetch();
		
		if($category['id'])
		{
			$category['productcount'] = getCategoryProductCount($category['id']);
			$category['populatecount'] = getCategoryAutoPopulateCount($category['id']);
		}
		
		return ($category);
	}
	
	function getCategoryProductCount($category_id, $all = true)
	{
		global $db;
		
		$category_ids[] = $category_id;
		$category_ids = array_merge($category_ids, getCategoryChildrenIDs($category_id));
		
		$category_ids_string = implode(',',$category_ids);
		
		$category_id = $db->cleanValue($category_id);
		
		if ($all)
		{
			$sql = "SELECT COUNT(DISTINCT(product_id)) FROM products_categories WHERE category_id IN ($category_ids_string)";
		}
		else
		{
			$sql = "SELECT COUNT(DISTINCT(p.id)) FROM products p, products_categories c
					WHERE p.id = c.product_id
					  AND c.category_id IN ($category_ids_string)
					  AND p.hidden = 0
					  AND p.enabled = 1
					  AND p.offers > 0
					  AND p.parent_id = 0";
		}
		
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		return($count);
	}
	
	function getCategoryAutoPopulateCount($category_id)
	{
		global $db;
		
		$sql = "SELECT COUNT(DISTINCT(id)) FROM categories_autopopulate WHERE category_id = '{$category_id}'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		return($count);
	}
	
	
	
	function getCategoryChildrenIDs($parent_id)
	{
		global $db;
		
		$sql = "SELECT id FROM categories WHERE parent_id = '{$parent_id}'";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		if (!$ra) return array();
		
		$category_ids = array();
		foreach ($ra as $item)
		{
			$category_ids[] = $item['id'];
			$category_ids = array_merge($category_ids, getCategoryChildrenIDs($item['id']));
		}
		
		return ($category_ids);
	}


	function getCategoryChildren($parent_id = 0, $exclude_id = 0)
	{
		global $db;
		
		$parent_id = $db->cleanValue($parent_id);
		$exclude_id = $db->cleanValue($exclude_id);
		$sql = "SELECT * FROM categories WHERE parent_id = '{$parent_id}' AND id <> '{$exclude_id}' ORDER BY sortorder ASC";
		$rs = $db->query($sql);
		if ($rs->rows() == 0) return false;
		$ra = $rs->getFullArray();
		
		$children = array();
		foreach ($ra as $child)
		{
			$child['children'] = getCategoryChildren($child['id'], $exclude_id);
			$children[] = $child;
		}
	
		return ($children);
	}
	
	function getCategoryList($parent_id = 0, $level = 0, $thislevelonly = false, $categorygroup_id = false, $subcategoriesof = false)
	{
		global $db, $amazon;
	
		$parent_id = $db->cleanValue($parent_id);
		
		if ($categorygroup_id !== false && $level == 0)
		{
			$sql = "SELECT * FROM categories WHERE parent_id = '{$parent_id}' AND categorygroup_id = '{$categorygroup_id}' ORDER BY categorygroup_id, sortorder ASC";
		}
		else
		{
			$sql = "SELECT * FROM categories WHERE parent_id = '{$parent_id}' ORDER BY categorygroup_id, sortorder ASC";
		}
		$rs = $db->query($sql);
		if ($rs->rows() == 0) return array();
		$ra = $rs->getFullArray();
		
		
		$categories = array();
		foreach ($ra as $category)
		{
			$category['level'] = $level;
			$category['productcount'] = getCategoryProductCount($category['id']);
			$category['populatecount'] = getCategoryAutoPopulateCount($category['id']);
			$category['browsenodelink'] = $amazon->createBrowseNodeLink($category['browsenode']);
			$category['searchtermlink'] = $amazon->createSearchTermLink($category['searchterm']);
			
			$category['categorygroup'] = $db->get('categorygroups', $category['categorygroup_id']);
			
			$categories[] = $category;
			
			if (!$thislevelonly || $subcategoriesof == $category['id'])
			{
				$newcategories = getCategoryList($category['id'], $level+1);
				if (sizeof($newcategories) > 0) $categories = array_merge($categories, $newcategories);
			}
				
		}
		
			
		return ($categories);
	}
	
	function getCategoryListByGroup($parent_id = 0, $level = 0, $thislevelonly = false, $subcategoriesof = false)
	{
		global $db;
		
		$groups = $db->query("SELECT * FROM categorygroups ORDER BY sortorder ASC")->getFullArray();
		
		$return = array();
		
		foreach ($groups as $group)
		{
			$return[$group['id']] = $group;
			$return[$group['id']]['categories'] = getCategoryList($parent_id, $level, $thislevelonly, $group['id'], $subcategoriesof);
		}
		
		return ($return);
	}
	
	
	
	
	
	function generateCategoryFormOptions($categorytree, $selected = 0, $level = 0, $first = true)
	{
		if (!is_array($selected))
		{
			$temp = $selected;
			$selected = array();
			$selected[] = $temp;
		}
	
	
		$options = '';
		if ($first)
		{
			$options .= '<option value="0"'.($selected == 0 ? ' selected' : '').'>-- none --</option>';
		}
		foreach ($categorytree as $item)
		{
			$options .= '<option value="'.$item['id'].'"'.(in_array($item['id'], $selected) ? 'selected' : '').'>';
			for ($i=0;$i<$level;$i++)
			{
				$options .= '-';
			}
			$options .= $item['linktitle'];
			$options .= '</option>';
			if ($item['children']) $options .= generateCategoryFormOptions($item['children'], $selected, $level+1, false);
		}
		
		return $options;
	}
	
	function deleteCategory($category_id)
	{
		global $db;
		
		# Redirect the category
		$category = getCategory($category_id);
		
		if ($category['parent_id'] == 0)
		{
			$redirect_to = '';
		}
		else
		{
			$parent = getCategory($category['parent_id']);
			$redirect_to = $parent['pagename'];
		}
		setPageRedirect($category['pagename'], $redirect_to);

		deleteCategoryProductLinks($category_id);
		deleteCategoryAutopopulateSettings($category_id);
				
		$category_id = $db->cleanValue($category_id);
		$sql = "DELETE FROM categories WHERE id = $category_id";
		$db->query($sql);
	}	

	function deleteCategoryProductLinks($category_id)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		$sql = "DELETE FROM products_categories WHERE category_id = '{$category_id}'";
		$db->query($sql);
	}
	
	
	
	function getCategoryPathsByProduct($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		
		$sql = "SELECT category_id FROM products_categories WHERE product_id = '{$product_id}'";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray(true);
		
		
		$paths = array();
		foreach ($ra as $category_id)
		{
			$temppath = array();
			$nextcategory = $category_id;
			
			while ($nextcategory)
			{
				$temppath[] = $nextcategory;
				
				$sql = "SELECT parent_id FROM categories WHERE id = '{$nextcategory}'";
				$rs = $db->query($sql);
				$nextcategory = $rs->getSingleResult();
			}
			
			$paths[] = array_reverse($temppath);
		}
		
		
		return ($paths);
	}
	
	
	function getCategoryPathByCategory($category_id)
	{
		global $db;
		$category_id = $db->cleanValue($category_id);
		
		$sql = "SELECT parent_id FROM categories WHERE id = '{$category_id}'";
		$rs = $db->query($sql);
		$parent_id = $rs->getSingleResult();
		
		$paths = array();
			
		while ($parent_id != 0)
		{
			$paths[] = $parent_id;
			
			$sql = "SELECT parent_id FROM categories WHERE id = '{$parent_id}'";
			$rs = $db->query($sql);
			$parent_id = $rs->getSingleResult();
		}
			
		$paths = array_reverse($paths);
		
		return ($paths);
	}
	
	function getTopLevelCategoryId($category_id)
	{
		global $db;
		
		$parent_id = $db->value('categories', 'parent_id', $category_id);
		
		if ($parent_id != 0)
		{
			$category_id = getTopLevelCategoryId($parent_id);
		}
		
		return $category_id;
	}
	
	function hasSubCategories($category_id)
	{
		global $db;
		
		$ra = $db->query("SELECT * FROM categories WHERE parent_id = '{$category_id}'")->getFullArray();
		if (count($ra) > 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}	

?>
