<?php

	function checkBrowseNodeAutoUpdateCategories()
	{
		global $db, $amazon;
		
		$sql = "SELECT *
				FROM categories_autopopulate
				WHERE (TIMESTAMPDIFF (MINUTE, lastpopulated, NOW()) >= ".AUTO_POPULATE_DELAY_BROWSENODE." OR lastpopulated = 0)
				  AND populationtype = 'browsenode'
				  AND paused = 0
				ORDER BY lastpopulated ASC
				LIMIT 1";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		foreach ($ra as $key => $autopopulate)
		{
			if($autopopulate['category_id']) $category = getCategory($autopopulate['category_id']);
			if(!$category)
			{
				# No point in keeping the autopopulate setting if it has no category
				$db->delete("categories_autopopulate", $autopopulate['id']);
				continue;
			}
			
			$return = $amazon->getProductsByBrowseNode($autopopulate['browsenode'], $autopopulate['searchindex'], $autopopulate['lastpopulatedpage'], $autopopulate['merchantid'], $autopopulate['minprice'], $autopopulate['maxprice']);
			
			$products = $return['products'];
			$lastpopulatedpage = $return['page'];
			
			# Update the "lastpopulated" soon as possible to prevent multiple updating
			$sql = "UPDATE categories_autopopulate SET lastpopulatedpage = '{$lastpopulatedpage}', lastpopulated = NOW() WHERE id = '{$autopopulate['id']}'";
			$db->query($sql);
			
			$added_products = 0;
			foreach ($products as $product)
			{
				if (!filterAutoUpdateProduct($product, $autopopulate))
				{
					$added_products++;
					$product_id = saveAmazonProduct($product);
					
					# Update the Offers
					$savedproduct = $db->get('products', $product_id);
					updateProductOffers($savedproduct);
					
					# Is this product prevented from being in this category?
					$rs = $db->query("SELECT COUNT(*) FROM products_categories_banned WHERE product_id = $product_id AND category_id = '{$category['id']}'");
					$categorybanned = $rs->getSingleResult();
					if ($categorybanned < 1)
					{
						addProductToCategory($product_id, $category['id']);
					}
				}
			}			
					
			# Reset if all pages are done
			if($lastpopulatedpage == 10)
			{
				$sql = "UPDATE categories_autopopulate SET lastpopulatedpage = 0 WHERE id = '{$autopopulate['id']}'";
				$db->query($sql);
			}
			
			# Update products added count
			addAutoPopulateProductCount($autopopulate['id'], $added_products);
			
		}
	}
	
	function checkSearchAutoUpdateCategories()
	{
		global $db, $amazon;
		
		$sql = "SELECT *
				FROM categories_autopopulate
				WHERE (TIMESTAMPDIFF (MINUTE, lastpopulated, NOW()) >= ".AUTO_POPULATE_DELAY_SEARCH." OR lastpopulated = 0)
				  AND populationtype = 'search'
				  AND paused = 0
				ORDER BY lastpopulated ASC
				LIMIT 1";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
				
		foreach ($ra as $key => $autopopulate)
		{
			if($autopopulate['category_id']) $category = getCategory($autopopulate['category_id']);
			if(!$category)
			{
				# No point in keeping the autopopulate setting if it has no category
				$db->delete("categories_autopopulate", $autopopulate['id']);
				continue;
			}
		
			$return = $amazon->getProductsBySearch($autopopulate['searchindex'], $autopopulate['searchterm'], $autopopulate['lastpopulatedpage'], $autopopulate['merchantid'], $autopopulate['minprice'], $autopopulate['maxprice']);
			
			$products = $return['products'];
			$lastpopulatedpage = $return['page'];
			
			# Update the "lastpopulated" soon as possible to prevent multiple updating
			$sql = "UPDATE categories_autopopulate SET lastpopulatedpage = $lastpopulatedpage, lastpopulated = NOW() WHERE id = ".$autopopulate['id'];
			$db->query($sql);
			
			$added_products = 0;
			foreach ($products as $product)
			{	
				if (!filterAutoUpdateProduct($product, $autopopulate))
				{
					$added_products++;
					$product_id = saveAmazonProduct($product);
					
					# Update the Offers
					$savedproduct = $db->get('products', $product_id);
					updateProductOffers($savedproduct);
	
					# Is this product prevented from being in this category?
					$rs = $db->query("SELECT COUNT(*) FROM products_categories_banned WHERE product_id = '{$product_id}'' AND category_id = '{$category['id']}'");
					$categorybanned = $rs->getSingleResult();
					if ($categorybanned < 1)
					{
						addProductToCategory($product_id, $category['id']);
					}
				}
			}
				
			# Reset if all pages are done
			if($lastpopulatedpage == 10)
			{
				$sql = "UPDATE categories_autopopulate SET lastpopulatedpage = 0 WHERE id = '{$autopopulate['id']}'";
				$db->query($sql);
			}
			
			# Update products added count
			addAutoPopulateProductCount($autopopulate['id'], $added_products);

		}
		
	}
	
	function forceAutoPopulate($category, $autopopulate = false)
	{
		global $amazon, $db;
		
		if(!$autopopulate)
		{
			$autopopulate = getNextCategoryAutoPopulate($category['id']);
		}
		if(!$autopopulate)
		{
			return;
		}
		
		if ($autopopulate['populationtype'] == 'browsenode')
		{
			$return = $amazon->getProductsByBrowseNode($autopopulate['browsenode'], $autopopulate['searchindex'], $autopopulate['lastpopulatedpage'], $autopopulate['merchantid'], $autopopulate['minprice'], $autopopulate['maxprice']);
		}
		else if ($autopopulate['populationtype'] == 'search')
		{	
			$return = $amazon->getProductsBySearch($autopopulate['searchindex'], $autopopulate['searchterm'], $autopopulate['lastpopulatedpage'], $autopopulate['merchantid'], $autopopulate['minprice'], $autopopulate['maxprice']);
		}
		else
		{
			return false;
		}
		
		$products = $return['products'];
		$lastpopulatedpage = $return['page'];
		$returnproduct_ids = array();
		
		# Update the "lastpopulated" soon as possible to prevent multiple updating
		$sql = "UPDATE categories_autopopulate SET lastpopulatedpage = '{$lastpopulatedpage}', lastpopulated = NOW() WHERE id = '{$autopopulate['id']}'";
		$db->query($sql);
		
		$added_products = 0;
		foreach ($products as $product)
		{
			$filter_reason = filterAutoUpdateProduct($product, $autopopulate);
			if (!$filter_reason)
			{
				$added_products++;
				$product_id = saveAmazonProduct($product);
				$returnproduct_ids[] = $product_id;
				
				# Is this product prevented from being in this category?
				$rs = $db->query("SELECT COUNT(*) FROM products_categories_banned WHERE product_id = '{$product_id}' AND category_id = '{$category['id']}'");
				$categorybanned = $rs->getSingleResult();
				if ($categorybanned < 1)
				{
					addProductToCategory($product_id, $category['id']);
				}
				
				# Update the offers
				$thisproduct = $db->get('products', $product_id);
				updateProductOffers($thisproduct, true);

				# Update the reviews
				updateProductReviews($thisproduct);
			}
			else
			{
				# Just save the ASIN so that details can be shown later
				$returnproduct_ids[] = $product;
				$returnproduct_ids[count($returnproduct_ids)-1]['filter_reason'] = $filter_reason;
			}
		}
		
		# Reset if all pages are done
		if($lastpopulatedpage == 10)
		{
			$sql = "UPDATE categories_autopopulate SET lastpopulatedpage = 0 WHERE id = '{$autopopulate['id']}'";
			$db->query($sql);
		}
		
		# Update products added count
		addAutoPopulateProductCount($autopopulate['id'], $added_products);
		
		return ($returnproduct_ids);
	}
	
	function filterAutoUpdateProduct($product, $autopopulate)
	{
		global $db;
		# Keyword Filtering
		
		$filter_reason = false;
		$requiredkeywords = ($autopopulate['requiredkeywords'] == '' ? false : explode(",", $autopopulate['requiredkeywords']));
		$filteredkeywords = ($autopopulate['filteredkeywords'] == '' ? false : explode(",", $autopopulate['filteredkeywords']));
		$requiredtype = $autopopulate['requiredkeywordsmatch'];
		$filteredtype = $autopopulate['filteredkeywordsmatch'];
		
		#Check the required keywords for blanks
		foreach ($requiredkeywords as $key => $item)
		{
			if (trim($item) == '') unset($requiredkeywords[$key]);
		}
		
		# Set up the description, if needed
		if($autopopulate['filterdescription'] == 1)
		{
			$product['core']['title'] .= " ".$product['core']['description']." ".$product['core']['keyfeatures'];
		}
		
		if ($requiredkeywords && $requiredtype == 'and')
		{
			$valid = true;
			foreach ($requiredkeywords as $key => $item)
			{
				if (strpos(strtolower($product['core']['title']), strtolower($item)) === false)
				{
					$valid = false;
					$filter_reason = "Missing required keyword: {$item}";
				}
			}
			if (!$valid) return $filter_reason;
		}
		
		if ($requiredkeywords && $requiredtype == 'or')
		{
			$valid = false;
			foreach ($requiredkeywords as $key => $item)
			{
				if (strpos(strtolower($product['core']['title']), strtolower($item)) !== false)
				{
					$valid = true;
				}
			}
			if (!$valid)
			{
				$filter_reason = "Missing any required keyword(s)";
				return $filter_reason;
			}
		}
		
		
		if ($filteredkeywords && $filteredtype == 'or')
		{
			$valid = true;
			foreach ($filteredkeywords as $key => $item)
			{
				if (strpos(strtolower($product['core']['title']), strtolower($item)) !== false)
				{
					$valid = false;
					$filter_reason = "Has filtered keyword: {$item}";
				}
			}
			
			if (!$valid) return $filter_reason;
		}
		
		if ($filteredkeywords && $filteredtype == 'and')
		{
			$valid = false;
			foreach ($filteredkeywords as $key => $item)
			{
				if (strpos(strtolower($product['core']['title']), strtolower($item)) === false)
				{
					$valid = true;
				}
			}
			
			if (!$valid)
			{
				$filter_reason = "Has all filtered keyword(s)";
				return $filter_reason;
			}
		}
		
		# Check If Product Already Exists
		$originalproduct = $db->get('products', $product['core']['asin'], 'asin');
		if($originalproduct['id'])
		{
			if(AUTO_POPULATE_EXISTING == 1)
			{
				# Add the product to the new category
				addProductToCategory($originalproduct['id'], $autopopulate['category_id']);
			}
			else
			{
				$filter_reason = "Product already added";
				return $filter_reason;
			}
		}
		
		# Maximum Salesrank
		if ($autopopulate['maxsalesrank'] > 0 && ($product['core']['salesrank'] >= $autopopulate['maxsalesrank'] || $product['core']['salesrank'] == 0 ))
		{
			$filter_reason = "Sales rank under {$autopopulate['maxsalesrank']}";
			return $filter_reason;
		}
		
		# Minimum Images Number
		if ((!$product['images'] && $autopopulate['minimages']) || count($product['images']) < $autopopulate['minimages'])
		{
			$filter_reason = "Image count under {$autopopulate['minimages']}";
			return $filter_reason;
		}
		
		
		# Not filtered
		return false;
	}
	
	function getAutoPopulate($autopopulate_id)
	{
		global $db;
		
		$autopopulate_id = $db->cleanValue($autopopulate_id);
		$sql = "SELECT * FROM categories_autopopulate WHERE id = '{$autopopulate_id}'";
		$rs = $db->query($sql);
		$autopopulate = $rs->fetch();
		
		return ($autopopulate);
	}
	
	function getCategoryAutoPopulates($category_id)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		$sql = "SELECT * FROM categories_autopopulate WHERE category_id = '{$category_id}'";
		$rs = $db->query($sql);
		$autopopulates = $rs->getFullArray();
		
		return ($autopopulates);
	}
	
	function getCategoryNextAutoPopulate($category_id)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		$sql = "SELECT * FROM categories_autopopulate WHERE category_id = '{$category_id}' AND populationtype <> 'cancel' AND populationtype <> '' ORDER BY lastpopulated";
		$rs = $db->query($sql);
		$autopopulate = $rs->fetch();
		
		return ($autopopulate);
	}
	
	function addAutoPopulateProductCount($autopopulate_id, $product_count)
	{
		global $db;
		
		if($product_count != 0)
		{
			$sql = "UPDATE categories_autopopulate SET productsadded = productsadded + {$product_count} WHERE id = {$autopopulate_id}";
			$db->query($sql);

			$sql = "UPDATE categories_autopopulate SET paused = 1 WHERE id = '{$autopopulate_id}' AND productsadded >= productlimit AND productlimit > 0";
			$db->query($sql);
		}
	}

	function deleteCategoryAutopopulateSettings($category_id)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		$sql = "DELETE FROM categories_autopopulate WHERE category_id = '{$category_id}'";
		$db->query($sql);
	}
	
		
?>
