<?php

	function getManufacturer($manufacturer_id)
	{
		global $db;
		
		$manufacturer = $db->get('manufacturers', $manufacturer_id);
		$manufacturer['productcount'] = getManufacturerProductCount($manufacturer_id);
		
		return ($manufacturer);
	}
	
	function getTopManufacturersByProductCount($minproducts = 0)
	{
		global $db;
		
		$sql = "SELECT m.*, COUNT(p.id) as productcount FROM manufacturers m, products p WHERE m.id = p.manufacturer_id AND m.merge_manufacturer_id = 0 GROUP BY m.id HAVING COUNT(p.id) > $minproducts ORDER BY COUNT(p.id) DESC";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$manufacturers = array();
		foreach($ra as $key => $item)
		{
			$manufacturers[$item['id']] = $item;
		}
		
		return ($manufacturers);
		
	}

	function getManufacturerList($exclude = array(0), $excludemerged = false)
	{
		global $db;
		
		if (!$exclude || sizeof($exclude) == 0) $exclude = array(0);
		$exclude = implode(',',$exclude);
		
		$sql = "SELECT * FROM manufacturers WHERE id NOT IN ($exclude)";
		if ($excludemerged) $sql = $sql . " AND merge_manufacturer_id = 0";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$manufacturers = array();
		foreach ($ra as $key => $item)
		{
			$manufacturer = $item;
			$manufacturer['productcount'] = getManufacturerProductCount($item['id']);			
			$manufacturers[$item['id']] = $manufacturer;
		}
		$manufacturers = sortArrayByField($manufacturers, 'name');
		
		return ($manufacturers);
	}

	function getManufacturerProductCount($manufacturer_id)
	{
		global $db;
		$manufacturer_id = $db->cleanValue($manufacturer_id);
		
		$sql = "SELECT COUNT(*) FROM products WHERE manufacturer_id = '{$manufacturer_id}'";
		$rs = $db->query($sql);
		return ($rs->getSingleResult());
	}


	function validateManufacturer($manufacturer)
	{
		if (!isset($manufacturer['linktitle']) || $manufacturer['linktitle'] == '' )
		{
			setMessage('error', 'Manufacturer title is required');
			return false;
		}
		
		if (!isset($manufacturer['name']) || $manufacturer['name'] == '' )
		{
			setMessage('error', 'Manufacturer name is required');
			return false;
		}
		
		if (strlen($manufacturer['metadescription']) > 180)
		{
			setMessage('error', 'Meta description can be no longer than 180 characters. (Currently ' . strlen($manufacturer['metadescription']) . ' characters)');
			return false;
		}
		
		return true;
	}


	function saveManufacturer($manufacturer_id, $manufacturer)
	{
		global $db;
		
		if (!validateManufacturer($manufacturer))
		{
			return false;
		}
	
		$manufacturer = generateManufacturerFields($manufacturer, $manufacturer_id);
				
		# Check for a pagename change, insert into legacy
		$oldpagename = $db->value('manufacturers', 'pagename', $manufacturer_id);
		if (isset($manufacturer['pagename']) && $oldpagename != $manufacturer['pagename'])
		{
			setPageRedirect($oldpagename, $manufacturer['pagename']);
		}
		
		
		$db->update('manufacturers', $manufacturer_id, $manufacturer);
	
		return true;
	}






	function manufacturerExists($amazonname)
	{
		global $db;
		$amazonname = $db->cleanValue($amazonname);		
		$sql = "SELECT id FROM manufacturers WHERE amazonname = '$amazonname'";
		$rs = $db->query($sql);
		$id = $rs->getSingleResult($rs);
		if (!$id) return false;
		else return $id;
	}
	
	
	function addProductToManufacturer($product_id, $amazonname)
	{
		global $db;
		
		$manufacturer_id = manufacturerExists($amazonname);
		
		if (!$manufacturer_id)
		{	
			$manufacturer['amazonname'] = $amazonname;
			$manufacturer['name'] = formatText($amazonname);
			$manufacturer['linktitle'] = $manufacturer['name'];
			$manufacturer['pageheading'] = $manufacturer['name'];
			$manufacturer = generateManufacturerFields($manufacturer);
			
			$manufacturer_id = $db->insert('manufacturers', $manufacturer);
		}
		
		$sql = "UPDATE	products
				SET		original_manufacturer_id = '{$manufacturer_id}',
						manufacturer_id = CASE manufacturer_id WHEN original_manufacturer_id THEN '{$manufacturer_id}' WHEN 0 THEN '{$manufacturer_id}' ELSE manufacturer_id END
				WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	function generateManufacturerFields($manufacturer, $manufacturer_id = 0)
	{		
		if (!isset($manufacturer['pagename']) || $manufacturer['pagename'] == '')
		{
			$manufacturer['pagename'] = createPageName(array($manufacturer['name']), array('manufacturer_id' => $manufacturer_id));
		}
		else
		{
			$manufacturer['pagename'] = getUniquePageName($manufacturer['pagename'], array('manufacturer_id' => $manufacturer_id));
		}
		
		if (!isset($manufacturer['metatitle']) || $manufacturer['metatitle'] == '')
			$manufacturer['metatitle'] = $manufacturer['name'] . ' - ' . SITE_NAME;
			
		if (!isset($manufacturer['pageheading']) || $manufacturer['pageheading'] == '')
			$manufacturer['pageheading'] = $manufacturer['name'];
			
		if (!isset($manufacturer['metadescription']) || $manufacturer['metadescription'] == '')
		{
			if (!isset($manufacturer['description']) || $manufacturer['description'] == '')
			{
				$manufacturer['metadescription'] = 'Buy ' . $manufacturer['name'] . ' securely online today at a great price. ' . $manufacturer['name'] . ' available today at ' . SITE_NAME.'.';
			}
			else
			{
				$manufacturer['metadescription'] = substr(strip_tags($manufacturer['description']),0,180);
			}
			
			# Make sure it isn't too big
			$size = 180;
			
			if (strlen($manufacturer['metadescription']) > $size)
			{
				$product['metadescription'] = substr($manufacturer['metadescription'], 0, $size-3).'...';
			}
		}
		
		if (!isset($manufacturer['metakeywords']) || $manufacturer['metakeywords'] == '')
		{
			$dataforkeywords[] = $manufacturer['keyfeatures'];
			$dataforkeywords[] = $manufacturer['name'];
			$manufacturer['metakeywords'] = createKeywordString($dataforkeywords);
		}
		
		return ($manufacturer);
	}
	

	
	
	function generateManufacturerFormOptions($manufacturerlist, $selected = 0, $first = true)
	{
		$options = '';
		if ($first)
		{
			$options .= '<option value="0"'.($selected == 0 ? ' selected' : '').'>-- none --</option>';
		}
		foreach ($manufacturerlist as $item)
		{
			$options .= '<option value="'.$item['id'].'"'.($item['id'] == $selected ? 'selected' : '').'>';
			$options .= $item['name'] . ' ('.$item['productcount'].')';
			$options .= '</option>';
		}
		
		return $options;
	}
	
	function mergeManufacturer($from_id, $to_id)
	{
		global $db;
		$from_id = $db->cleanValue($from_id);
		$to_id = $db->cleanValue($to_id);
		
		$sql = "UPDATE manufacturers SET merge_manufacturer_id = '{$to_id}' WHERE id = '{$from_id}'";
		$db->query($sql);
		
		$sql = "UPDATE products SET manufacturer_id = '{$to_id}' WHERE original_manufacturer_id = '{$from_id}'";
		$db->query($sql);			
	}
	
	function cancelManufacturerMerge($manufacturer_id)
	{
		global $db;
		$manufacturer_id = $db->cleanValue($manufacturer_id);
		
		$sql = "UPDATE manufacturers SET merge_manufacturer_id = 0 WHERE id = '{$manufacturer_id}'";
		$db->query($sql);
		
		$sql = "UPDATE products SET manufacturer_id = original_manufacturer_id WHERE original_manufacturer_id = '{$manufacturer_id}'";
		$db->query($sql);			
	}
	
	function displayManufacturers($manufacturer_ids, $display_type)
	{
		changeManufacturerDisplayStates($manufacturer_ids, $display_type, 1);
	}
	
	function hideManufacturers($manufacturer_ids, $display_type)
	{
		$display_type = str_replace('hide', 'display', $display_type);
		changeManufacturerDisplayStates($manufacturer_ids, $display_type, 0);
	}
	
	function changeManufacturerDisplayStates($manufacturer_ids, $display_type, $display_value)
	{
		global $db;
		
		if($display_type === true)
		{
			$set = "display_sidebar = {$display_value}, display_allmanufacturers = {$display_value}, display_product = {$display_value}, display_homepage = {$display_value}, display_filter = {$display_value}";
		}
		else $set = "{$display_type} = {$display_value}";
		
		foreach($manufacturer_ids as $manufacturer_id)
		{
			$manufacturer_id = $db->cleanValue($manufacturer_id);
			$sql = "UPDATE manufacturers SET {$set} WHERE id = '{$manufacturer_id}'";
			$db->query($sql);
		}
	}
?>
