<?php

	function redirect($to)
	{
		header("HTTP/1.1 301 Moved Permanently"); 
		header("Location: $to");
		exit();
	}
	
	function sortArrayByField($original,$field,$descending = false)
	{
		$sortArr = array();
		
		foreach ($original as $key => $value)
		{
			$sortArr[$key] = $value[$field];
		}
		
		if ($descending)
		{
			arsort($sortArr);
		}
		else
		{
			asort($sortArr);
		}
		
		$resultArr = array();
		foreach ($sortArr as $key => $value)
		{
			$resultArr[$key] = $original[$key];
		}
		
		return $resultArr;
	}
	
	function stripslashesDeep($value)
	{
		$value = is_array($value) ?
					array_map('stripslashesDeep', $value) :
					stripslashes($value);
	
		return $value;
	}

	function cleanGPC()
	{	
		
		if ((function_exists("get_magic_quotes_gpc") && get_magic_quotes_gpc()) || (ini_get('magic_quotes_sybase') && (strtolower(ini_get('magic_quotes_sybase'))!="off")))
		{
			$_GET = stripslashesDeep($_GET);
			$_POST = stripslashesDeep($_POST);
			$_COOKIE = stripslashesDeep($_COOKIE);
		}
	}
	
	function removeNewLines($string)
	{
		$string = str_replace(chr(10), " ", $string);
		$string = str_replace(chr(13), " ", $string);
		return ($string);
	}
	
	function isCommonWord($word)
	{
		# http:#www.textfixer.com/
		$commonwords = explode(',', "with,when,from,today,using,'tis,'twas,a,able,about,across,after,ain't,all,almost,also,am,among,an,and,any,are,aren't,as,at,be,because,been,but,by,can,can't,cannot,could,could've,couldn't,dear,did,didn't,do,does,doesn't,don't,either,else,ever,every,for,from,get,got,had,has,hasn't,have,he,he'd,he'll,he's,her,hers,him,his,how,how'd,how'll,how's,however,i,i'd,i'll,i'm,i've,if,in,into,is,isn't,it,it's,its,just,least,let,like,likely,may,me,might,might've,mightn't,most,must,must've,mustn't,my,neither,no,nor,not,of,off,often,on,only,or,other,our,own,rather,said,say,says,shan't,she,she'd,she'll,she's,should,should've,shouldn't,since,so,some,than,that,that'll,that's,the,their,them,then,there,there's,these,they,they'd,they'll,they're,they've,this,tis,to,too,twas,us,wants,was,wasn't,we,we'd,we'll,we're,were,weren't,what,what'd,what's,when,when,when'd,when'll,when's,where,where'd,where'll,where's,which,while,who,who'd,who'll,who's,whom,why,why'd,why'll,why's,will,with,won't,would,would've,wouldn't,yet,you,you'd,you'll,you're,you've,your");		
		return in_array($word, $commonwords);
	}
	
	function createKeywordString($content = array(), $maxlength = 3)
	{
		$keywords = array();
		
		foreach ($content as $item)
		{
			# Clean up the string
			$item = strip_tags($item);
			$item = strtolower($item);
			$item = trim($item);
			$item = preg_replace('([^A-Za-z0-9-&\.\s]+)','',$item);
			$item = preg_replace('([\s]+)',' ',$item);
			$item = preg_replace('([-]{2,})','-',$item);
			$item = str_replace(' -', '', $item);
			$item = str_replace('- ', '', $item);
			$item = str_replace('&', 'and', $item);
			$item = trim($item);

			# Move on if it's empty after all of this cleaning
			if ($item == '') continue;
			
			# Split it up into individual words
			$tempkeywords = explode(' ', $item);
			
			# Create the base keyword list by grouping the individual words into single, double, and triple keywords
			# For example:
			# The list of words [a, b, c, d] would be split up into:
			# [a, b, c, d, ab, bc, cd, abc, bcd]
			# Only adjacent words are grouped together
			for($i = 0; $i < count($tempkeywords); $i++)
			{
				# Single keyword
				$keyword = $tempkeywords[$i];
				$keywords[$keyword] += filterKeyword($keyword);
				
				# Double keyword
				if($maxlength >= 2) if($tempkeywords[$i+1])
				{
					$keyword = $tempkeywords[$i]." ".$tempkeywords[$i+1];
					$keywords[$keyword] += filterKeyword($keyword);
				}
				
				# Triple keyword
				if($maxlength >= 3) if($tempkeywords[$i+1] && $tempkeywords[$i+2])
				{
					$keyword = $tempkeywords[$i]." ".$tempkeywords[$i+1]." ".$tempkeywords[$i+2];
					$keywords[$keyword] += filterKeyword($keyword);
				}
			}
			
		}
		
		# Some cleaning up
		foreach($keywords as $keyword => $points)
		{
			# Remove any keywords that got added but didn't score any points
			if($points == 0)
			{
				unset($keywords[$keyword]);
				continue;
			}
			# Remove any common words
			elseif(isCommonWord($keyword))
			{
				unset($keywords[$keyword]);
				continue;
			}
			# Remove any numbers
			elseif(is_numeric($keyword))
			{
				unset($keywords[$keyword]);
				continue;
			}
			# Remove any dates
			elseif(strtotime($keyword))
			{
				unset($keywords[$keyword]);
				continue;
			}
			
			# Adjust the score if common words appear on the outside
			$keyword_parts = explode(" ", $keyword);
			if(count($keyword_parts) > 1)
			{
				$point_loss = 1;
				if(isCommonWord($keyword_parts[0])) $points -= $point_loss;
				if(isCommonWord($keyword_parts[count($keyword_parts)-1])) $points -= $point_loss;
				$keywords[$keyword] = $points;
			}
		}
		
		# Sort them correctly by ranking
		asort($keywords);
		$keywords = array_reverse($keywords, true);
		
		# Now, the fun begins, we have to go from a huge list of keywords down to 10
		# While selecting single, double, and triple keywords correctly
		foreach($keywords as $keyword1 => $score1)
		{
			foreach($keywords as $keyword2 => $score2)
			{
				if($keyword1 == $keyword2) continue;
				if(strpos($keyword1, $keyword2) !== false)
				{
					# One of the keywords is inside of another, we're going to have to get rid of one of them
					# It's pretty easy if one has a better score
					if($score1 > $score2)
					{
						unset($keywords[$keyword2]);
					}
					elseif($score2 > $score1)
					{
						unset($keywords[$keyword1]);
					}
					else
					{
						# But otherwise, let's just go with the longer one
						if(strlen($keyword1) > strlen($keyword2))
						{
							unset($keywords[$keyword2]);
						}
						else
						{
							unset($keywords[$keyword1]);
						}
					}
					continue;
				}
				
				# Split up the words and check if they both have three
				$keyword1_parts = explode(" ", $keyword1);
				$keyword2_parts = explode(" ", $keyword2);
				if(count($keyword1_parts) != 3 || count($keyword2_parts) != 3) continue;
				
				# This takes care of [a b c] and [b c d] which are too similar to have both
				$matches = 0;
				for($i = 0; $i < 3; $i++)
					$matches += (array_search($keyword2_parts[$i], $keyword1_parts) === false)? 0 : 1;
				
				# Have found the above mentioned case?
				if($matches == 2)
				{
					# This probably could be simplified, it's the same as above
					# One of the keywords is inside of another, we're going to have to get rid of one of them
					# It's pretty easy if one has a better score
					if($score1 > $score2)
					{
						unset($keywords[$keyword2]);
					}
					elseif($score2 > $score1)
					{
						unset($keywords[$keyword1]);
					}
					else
					{
						# But otherwise, let's just go with the longer one
						if(strlen($keyword1) > strlen($keyword2))
						{
							unset($keywords[$keyword2]);
						}
						else
						{
							unset($keywords[$keyword1]);
						}
					}
					continue;
				}
			}
		}
		
		# The final keywords will go here
		$finalkeywords = array();
		
		# Get the final keywords
		$i = 0;
		foreach($keywords as $keyword => $score)
		{
			$finalkeywords[] = $keyword;
			if(++$i == METAKEYWORD_COUNT) break;
		}
		
		return(implode(',',$finalkeywords));
	}
	
	function filterKeyword($keyword)
	{
		if($keyword == '' || strlen($keyword) <= 3 || strlen(str_replace(' ', '', $keyword)) <= 3) return 0;
		else return 1;
	}
	
	function validEmail($email)
	{
		if (preg_match("/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$/", $email))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	function splitStringByCaptialLetter($string)
	{
		return trim(preg_replace('/[A-Z][^A-Z]+/',"$0 ",$string));
	}
	
	function formatText($value)
	{
		# Removes dodgy characters, fix case, remove new lines
		$value = preg_replace('([^A-Za-z0-9-\s]+)','',$value);
		$value = removeNewLines(ucwords(strtolower($value)));
		$value = preg_replace("/[ ]{2,}/", ' ', $value);
		return ($value);
	}
	
	function buildListingLink($baselink, $page, $sort, $defaultsort, $perpage, $defaultperpage = PER_PAGE_LISTING)
	{
		$page = (int)$page;
		$perpage = (int)$perpage;
		
		if ($page == 0) $page = 1;
		
		if ($perpage != $defaultperpage)
		{
			return $baselink.$page.'/'.$sort.'/'.$perpage.'/';
		}
		else if ($sort != $defaultsort)
		{
			return $baselink.$page.'/'.$sort.'/';
		}
		else if ($page != 1)
		{
			return $baselink.$page.'/';
		}
		else
		{
			return $baselink;
		}
	}
	
	function makeArray($item)
	{
		if (!is_array($item))
		{
			$temp = $item;
			$item = array();
			$item[] = $temp;
		}
		return $item;
	}
	
	function simpleXmlToArray($simplexml)
	{
		$children = $simplexml->children();
		$arr = array();
		foreach ($children as $elementName => $node)
		{
			$nextIdx = count($arr);
			$arr[$nextIdx] = array();
			$arr[$nextIdx]['@name'] = strtolower((string)$elementName);
			$arr[$nextIdx]['@attributes'] = array();
			$attributes = $node->attributes();
			foreach ($attributes as $attributeName => $attributeValue)
			{
				$attribName = strtolower(trim((string)$attributeName));
				$attribVal = trim((string)$attributeValue);
				$arr[$nextIdx]['@attributes'][$attribName] = $attribVal;
			}
			$text = (string)$node;
			$text = trim($text);
			if (strlen($text) > 0)
			{
				$arr[$nextIdx]['@text'] = $text;
			}
			$arr[$nextIdx]['@children'] = array();
			simpleXmlToArray($node, $arr[$nextIdx]['@children']);
		}
		return $arr; 
	}
	
	function XMLObjectToArray($xml)
	{
	    $tree = null;
	    
	    if (!is_object($xml)) return false;
	    
	    while($xml->read())
	        switch ($xml->nodeType)
	        {
	            case XMLReader::END_ELEMENT: return $tree;
	            case XMLReader::ELEMENT:
	                $node = array('tag' => $xml->name, 'value' => $xml->isEmptyElement ? '' : XMLObjectToArray($xml));
	                if($xml->hasAttributes)
	                    while($xml->moveToNextAttribute())
	                        $node['attributes'][$xml->name] = $xml->value;
	                $tree[] = $node;
	            break;
	            case XMLReader::TEXT:
	            case XMLReader::CDATA:
	                $tree .= $xml->value;
	        }
	    return $tree;
	}
	
	function simplifyXMLArray($array)
	{
		$return = array();
		foreach ($array as $item)
		{
			if (is_array($item['value']))
			{
				if (isset($return[$item['tag']]) && (!is_array($return[$item['tag']]) || !$return[$item['tag']][0]))
				{
					$temp = $return[$item['tag']];
					$return[$item['tag']] = array();
					$return[$item['tag']][] = $temp;
					$return[$item['tag']][] = simplifyXMLArray($item['value']);
				}
				else if (is_array($return[$item['tag']]))
				{
					$return[$item['tag']][] = simplifyXMLArray($item['value']);
				}
				else
				{
					$return[$item['tag']] = simplifyXMLArray($item['value']);
				}
			}
			else
			{
				if (isset($return[$item['tag']]) && !is_array($return[$item['tag']]))
				{
					$temp = $return[$item['tag']];
					$return[$item['tag']] = array();
					$return[$item['tag']][] = $temp;
					$return[$item['tag']][] = $item['value'];
				}
				else if (is_array($return[$item['tag']]))
				{
					$return[$item['tag']][] = $item['value'];
				}
				else
				{
					$return[$item['tag']] = $item['value'];
				}
			}
		}
		
		return($return);
	}
	
	
	function getTimestamp($br = true, $tag = '')
	{
		list($usec, $sec) = explode(" ", microtime());
		return date (date("Y-m-d\TH:i:s ", $sec) . $usec ).' ' .$tag .($br ? '<br />' : '');
	}
	
	function getFooterText()
	{
		global $db;
		
		# Check if parameter exists
		if (!defined('FOOTER_TEXT_ID'))
		{
			#ÊGet a random footer text value
			$footertext_id = $db->query("SELECT id FROM footertext ORDER BY RAND() LIMIT 1")->getSingleResult();
			
		
			$insert['name'] = 'Footer Text';
			$insert['description'] = 'This is the text showing in the footer of your site, which links back to Fresh Store Builder. Leaving this really helps us out and does not affect your sites SEO performance. If you would prefer to remove it set this value to 0.';
			$insert['parameter'] = 'FOOTER_TEXT_ID';
			$insert['group'] = 'Store Settings';
			$insert['value'] = $footertext_id;
			
			$db->insert('parameters', $insert);
			
			define('FOOTER_TEXT_ID', $footertext_id);
		}
		
		$footertext = $db->query("SELECT footer FROM footertext WHERE id = ".FOOTER_TEXT_ID)->getSingleResult();
		
		return($footertext);
	}
	
	function CheckPermissions($path,$perm)
	{
	    clearstatcache();
		if( file_exists($path) ) {
			$configmod = substr(sprintf('%o', fileperms($path)), -4); 
			if ( $configmod == $perm ) return TRUE; else return FALSE;
		} else return TRUE;
	} 
	
	function parseForeignLanguageCharacters($text)
	{
		$chars = array();
		
		# German
		$chars['ü'] = '&uuml;';
		$chars['Ü'] = '&Uuml;';
		$chars['ë'] = '&euml;';
		$chars['Ë'] = '&Euml;';
		$chars['Ä'] = '&Auml;';
		$chars['ä'] = '&auml;';
		$chars['É'] = '&Eacute;';
		$chars['é'] = '&eacute;';
		$chars['Ö'] = '&Ouml;';
		$chars['ö'] = '&ouml;';
		$chars['ß'] = '&szlig;';
		
		foreach ($chars as $key => $item)
		{
			$text = str_replace($key, $item, $text);
		}
		
		return ($text);
	}
	
	function strip_word_html($text, $allowed_tags = '<b><i><sup><sub><em><strong><u><br>')
  {   
  	mb_regex_encoding('UTF-8');
    $search = array('/&lsquo;/u', '/&rsquo;/u', '/&ldquo;/u', '/&rdquo;/u', '/&mdash;/u');
    $replace = array('\'', '\'', '"', '"', '-');
    $text = preg_replace($search, $replace, $text);
    $text = html_entity_decode($text, ENT_QUOTES, 'UTF-8');
    if(mb_stripos($text, '/*') !== false)
    {
        $text = mb_eregi_replace('#/\*.*?\*/#s', '', $text, 'm');
    }
    $text = preg_replace(array('/<([0-9]+)/'), array('< $1'), $text);
    $text = strip_tags($text, $allowed_tags);
    $text = preg_replace(array('/^\s\s+/', '/\s\s+$/', '/\s\s+/u'), array('', '', ' '), $text);
    $search = array('#<(strong|b)[^>]*>(.*?)</(strong|b)>#isu', '#<(em|i)[^>]*>(.*?)</(em|i)>#isu', '#<u[^>]*>(.*?)</u>#isu');
    $replace = array('<b>$2</b>', '<i>$2</i>', '<u>$1</u>');
    $text = preg_replace($search, $replace, $text);
    $num_matches = preg_match_all("/\<!--/u", $text, $matches);
    if($num_matches)
    {
          $text = preg_replace('/\<!--(.)*--\>/isu', '', $text);
    }
    return $text;
  }
  
  function updateHtaccessFiles($subdirectory, $file_location = "../")
  {
  	foreach(array("{$file_location}.htaccess", "{$file_location}freshadmin/.htaccess") as $file_name)
  	{
  		# Check if we're doing freshadmin
  		if(strpos($file_name, "freshadmin") !== false) $subdirectory .= "freshadmin/";
  		
			# Open up the file
			$file_contents = @file_get_contents($file_name);
			$file_lines = explode("\n", $file_contents);
		
			# Try and find the line to replace
			$file_lines_count = count($file_lines);
			for($i = 0; $i < $file_lines_count; $i++)
			{
				$file_line = $file_lines[$i];
				if(strpos($file_line, "RewriteBase") !== false)
				{
					# We have found it
					$file_line = "RewriteBase {$subdirectory}";
					$file_lines[$i] = $file_line;
					break;
				}
			}
		
			# Generate the new file
			$file_contents = implode("\n", $file_lines);
		
			# Write the changes to the file
			if (function_exists('file_put_contents'))
			{
				file_put_contents($file_name, $file_contents);
			}
			else
			{
				$fh = fopen($file_name, 'w');
				fwrite($fh, $file_contents);
				fclose($fh);
			}
		}
  }

  function format_date($date, $since = false, $after = " ago", $databasetime = true)
  {
  	if(!is_numeric($date))
  	{
  		$time = strtotime($date);
  		if($databasetime) $time += MYSQL_TIMEZONE_OFFSET;
  	}
  	else $time = $date;
  	if($time < 0) return "never";

  	if($since && DATE_DISPLAY_SINCE)
  	{
  		$time = time() - $time;
  		$intervals = array(
  			"year" => 31556926,
  			"month" => 2629743,
  			"day" => 86400,
  			"hour" => 3600,
  			"minute" => 60,
  			"second" => 1
  		);
  		$return = array();

  		if($time == 0) $return[] = "0 seconds";
  		else foreach($intervals as $key => $value)
  		{
  			if($time >= $value)
  			{
  				$count = floor($time / $value);
  				$time %= $value;

  				$s = ($count != 1)? "s" : "";
  				$return[] = "{$count} {$key}{$s}";
  			}
  			if(count($return) >= $since) break;
  		}
  		return implode(", ", $return).$after;
  	}
  	else
  	{
  		return date(DATE_DISPLAY_FORMAT, $time);
  	}
  }

  function resetDatabaseTimeOffset()
  {
  	global $db;
  	
	  $date = $db->query("SELECT NOW()")->getSingleResult();
		$time = strtotime($date);
		$offset = time() - $time;
		$db->query("UPDATE parameters SET value = '{$offset}' WHERE parameter = 'MYSQL_TIMEZONE_OFFSET'");
  }
?>
