<?php
	
	function validatePage($page)
	{
		
		if (!isset($page['title']) || $page['title'] == '' )
		{
			setMessage('error', 'Page title is required');
			return false;
		}
		
		if (strlen($page['metadescription']) > 180)
		{
			setMessage('warning', 'Meta Description should be no longer than 180 characters. (Currently ' . strlen($page['metadescription']) . ' characters)');
		}
		
		return true;		
	}

	function addPage($page)
	{
		global $db;
		
		if (!validatePage($page))
		{
			return false;
		}
		
		# Fill in the gaps with auto generated info
		$page = generatePageFields($page);
		
		$page_id = $db->insert('pages', $page);
		
		return ($page_id);
	}
	
	function savePage($page_id, $page)
	{
		global $db;
		if (!validatePage($page))
		{
			return false;
		}

		# Fill in the gaps with auto generated info
		$page = generatePageFields($page, $page_id);
		
		# Check for a pagename change, insert into legacy
		$oldpagename = $db->value('pages', 'pagename', $page_id);
		if (isset($page['pagename']) && $oldpagename != $page['pagename'])
		{
			setPageRedirect($oldpagename, $page['pagename']);
		}
		
		$db->update('pages', $page_id, $page);
		
		return true;
	}
	
	function generatePageFields($page, $page_id = 0)
	{
	
		if ($page['pagekey'] != 'homepage')
		{
			if (!isset($page['pagename']) || $page['pagename'] == '')
			{
				$page['pagename'] = createPageName(array($page['title']), array('page_id' => $page_id));
			}
			else
			{
				$page['pagename'] = getUniquePageName($page['pagename'], array('page_id' => $page_id));
			}
		}
		
		if (!isset($page['pagekey']) || $page['pagekey'] == '') $page['pagekey'] = str_replace('-', '', $page['pagename']);
		
		if (!isset($page['metatitle']) || $page['metatitle'] == '')
			$page['metatitle'] = $page['title'];
			
		if (!isset($page['linktitle']) || $page['linktitle'] == '')
			$page['linktitle'] = $page['title'];
		
		if (!isset($page['metadescription']) || $page['metadescription'] == '')
		{
			if (!isset($page['description']) || $page['description'] == '')
			{
				$page['metadescription'] = $page['title'] . ' at ' . SITE_NAME.'.';
			}
			else
			{
				$page['metadescription'] = strip_tags($page['description']);
			}
			
			# Make sure it isn't too big
			if (strlen($page['metadescription']) > 180)
			{
				$page['metadescription'] = substr($page['metadescription'], 0, $size-3).'...';
			}
		}
		
		if (!isset($page['metakeywords']) || $page['metakeywords'] == '')
			$page['metakeywords'] = createKeywordString($page);
			
			
		# Update language characters on appropriate text
		#$page['maintext'] = parseForeignLanguageCharacters($page['maintext']);
		
		
		return ($page);
	}
	
	function getPageList()
	{
		global $db, $amazon;
	
		$sql = "SELECT * FROM pages";
		$rs = $db->query($sql);
		if ($rs->rows() == 0) return array();
		$pages = $rs->getFullArray();
		return ($pages);
	}
	
	function getPage($value, $field = 'id')
	{
		global $db;
		
		$value = $db->cleanValue($value);
		$sql = "SELECT * FROM pages WHERE $field = '$value'";
		$rs = $db->query($sql);
		$page = $rs->fetch();
				
		return ($page);
	}
	
	function deletePage($page_id)
	{
		global $db;
		$db->delete('pages', $page_id);
		$db->delete('sideboxes_pages', $page_id, 'page_id');
	}
	
	function getPagePagename($pagekey)
	{
		global $db;
		return $db->query("SELECT pagename FROM pages WHERE pagekey = '$pagekey'")->getSingleResult();
	}

?>