<?php

	
	# Gets the product and then adds it
	function addProductByASIN($asin)
	{
		global $amazon, $db;
		
		if (productExists($asin))
		{
			setMessage('error', 'This product ('.$asin.') has already been added.');
			return false;
		}
		
		$product = $amazon->getProductFull($asin);
		if (!$product)
		{
			setMessage('error', 'This ASIN ('.$asin.') was not found - please check and try again.');
			return false;
		}
		
		$product_id = saveAmazonProduct($product);		
		return ($product_id);
	}
	
	
	
	function saveAmazonProduct($product)
	{
		global $db;
		
		# Get original product informaton
		$originalproduct = $db->get('products', $product['core']['asin'], 'asin');
		$product_id = isset($originalproduct['id']) ? $originalproduct['id'] : 0;
		
		# Check Amazon Content
		if($product_id != 0) $originalamazoncontent = $db->get('productamazoncontent', $product_id, 'product_id');
		$product['core']['lockamazondescription'] = $originalamazoncontent['lockdescription']; # We may as well not mess around with it if it's going to end up locked
		
		# Take a copy of the Amazon data, then filter it
		$amazondata = array();
		$amazondata['originaltitle'] = $product['core']['title'];
		$amazondata['originaldescription'] = $product['core']['description'];
		$amazondata['originalkeyfeatures'] = $product['core']['keyfeatures'];
		$product['core'] = filterAmazonProductData($product['core']);
		$lockamazondescription = $product['core']['lockamazondescription'];
		unset($product['core']['lockamazondescription']);
		
		
		# Update amazondata with filtered data
		$amazondata['title'] = $product['core']['title'];
		$amazondata['description'] = $product['core']['description'];
		$amazondata['keyfeatures'] = $product['core']['keyfeatures'];
		
		# If its a new product or set to not edited manually, generate content
		$coreproduct = array();
		if (!$originalproduct || $originalproduct['edited'] == 0)
		{
			#Setup pagename, link title et al
			$coreproduct = generateProductFields($product['core'], $product_id);
			
			# Remove the current description from core product
			$coreproduct['description'] = '';
			
			# Generate content by templates
			$product['attributes']['Title'] = $amazondata['title'];
			$product['attributes'] = array_merge(getProductAttributesRaw($product_id), $product['attributes']); # Load up any custom product attributes
			$generatedcontent = generateContent($product['attributes'], $product_id);
			
			# The order here matters, this evaluates left to right and each new value overwrites the old
			# Content generated by user content templates gets priority, then content automatically
			# generated by the system, and then the Amazon product data
			if ($generatedcontent) $coreproduct = array_merge($product['core'], $coreproduct, $generatedcontent);
			
			#Mark as edited
			$coreproduct['edited'] = 1;
		}
		
		# Stuff from the core product we always want to update
		$coreproduct['url'] = $product['core']['url'];
		$coreproduct['parent'] = $product['core']['parent'];
		$coreproduct['salesrank'] = ($product['core']['salesrank'] == 0 ? 99999999999 : $product['core']['salesrank']);
		if($product['core']['marketplaceitem'] === 1) $coreproduct['marketplaceitem'] = $product['core']['marketplaceitem'];
			
		# Timestamps
		$coreproduct['productupdated'] = date('Y-m-d H:i:s');
		$coreproduct['added'] = date('Y-m-d H:i:s');
		
		$product_id = $originalproduct['id'];
		$amazondata['product_id'] = $product_id;
		
		if (!$product_id)
		{
			# Add Main Product
			$product_id = $db->insert('products', $coreproduct);
			
			#Add Amazon Content
			$db->insert('productamazoncontent', $amazondata);
		}
		else
		{
			# Update main product
			$db->update('products', $product_id, $coreproduct);
			
			if (empty($originalamazoncontent))
			{
				#Add Amazon Content
				$db->insert('productamazoncontent', $amazondata);
			}
			else
			{
				if ($originalamazoncontent['lockdescription'] == '1') unset($amazondata['description']);
				if ($lockamazondescription == '1') $amazondata['lockdescription'] = 1; # If this was locked somewhere in the process, let's commit the changes to the database now

				#Update Amazon Content
				$db->update('productamazoncontent', $product_id, $amazondata, 'product_id');				
			}

			# Check for pagename redirect
			if ($originalproduct['pagename'] != $coreproduct['pagename'])
			{
				setPageRedirect($originalproduct['pagename'], $coreproduct['pagename']);
			}
		}
		
		# Variations
		if ($product['core']['parent'] == 1) updateProductVariations($product_id, $product['core']['asin']);
		
		# Images
		saveProductImages($product_id, $product['images']);
		
		# Check if the Manufacturer is merged
		if (isManufacturerMerged($product['attributes']['Manufacturer'])) $product['attributes']['Manufacturer'] = isManufacturerMerged($product['attributes']['Manufacturer']);
		
		# Attributes
		saveProductAttributes($product_id, $product['attributes']);
		
		return $product_id;
	}
	
	#Function that returns if the manufacturer is merged
	function isManufacturerMerged($manName)
	{
		global $db;

		$manName = $db->cleanValue($manName);
		
		$rs = $db->query("SELECT merge_manufacturer_id FROM `manufacturers` WHERE amazonname = '{$manName}'");
		$ra = $rs->getFullArray();

		if ( $ra[0]["merge_manufacturer_id"] == "0" ) return FALSE; 
		else {
			$rs1 = $db->query('SELECT amazonname FROM `manufacturers` WHERE id = "'.$ra[0]["merge_manufacturer_id"].'"');
			$ra1= $rs1->getFullArray();
			return $ra1[0]["amazonname"];	 
		}
	}
	

	function filterAmazonProductData($product)
	{
		global $db;
		
		# HTML in the Amazon Description
		$product['description'] = html_entity_decode($product['description']);
		
		# Title in all capitals
		if (strtoupper($product['title']) == $product['title'])
		{
			$product['title'] = ucwords(strtolower($product['title']));
		}
		
		#Title filtered words (e.g. NEW!)
		$titlefilters = $db->query("SELECT `phrase`, `replace` FROM productphrasefilter WHERE `type` IN ('title', 'all') ORDER BY CHAR_LENGTH(phrase) DESC")->getFullArray();
		foreach ($titlefilters as $filter)
		{
			$product['title'] = str_ireplace($filter['phrase'], $filter['replace'], $product['title']);
		}
		
		#Remove double/triple spaces in title
		$product['title'] = str_replace('   ', ' ', $product['title']);
		$product['title'] = str_replace('  ', ' ', $product['title']);
		
		# Smart Quotes
		$product['description'] = str_replace('', "'", $product['description']);
		
		# Description filtered words
		$descfilters = $db->query("SELECT `phrase`, `replace` FROM productphrasefilter WHERE `type` IN ('description', 'all') ORDER BY CHAR_LENGTH(phrase) DESC")->getFullArray();
		foreach ($descfilters as $filter)
		{
			$product['description'] = str_ireplace($filter['phrase'], $filter['replace'], $product['description']);
		}
		
		#Remove double/triple spaces in title
		$product['description'] = str_replace('   ', ' ', $product['description']);
		$product['description'] = str_replace('  ', ' ', $product['description']);
		
		
		# Spin the content?
		# There's no point in spinning if it's locked
		if (BESTSPINNER_AUTOSPINAMAZON == 1 && !$product['lockamazondescription'])
		{
			# Create a template via best spinner, then spin locally
			$template = createBestSpinnerTemplate($product['description']);
			$spun = performContentSwitching($template);
			$spun = replaceContentTemplateConstants($spun);
			$product['description'] = $spun[0];
			
			# If it has been spun, we should lock it
			$product['lockamazondescription'] = 1;
		}
		$product['description'] = parseForeignLanguageCharacters($product['description']);

		return ($product);
	}

	function generateProductFields($product, $product_id = 0)
	{		
		if (!isset($product['pagename']) || $product['pagename'] == '')
		{
			$product['pagename'] = createPageName(array($product['title']), array('product_id' => $product_id));
		}
		else
		{
			$product['pagename'] = getUniquePageName($product['pagename'], array('product_id' => $product_id));
		}
		
		if (!isset($product['metatitle']) || $product['metatitle'] == '')
			$product['metatitle'] = $product['title'];
			
		if (!isset($product['linktitle']) || $product['linktitle'] == '')
			$product['linktitle'] = $product['title'];
		
		if (!isset($product['metadescription']) || $product['metadescription'] == '')
		{
			if (!isset($product['description']) || $product['description'] == '')
			{
				$product['metadescription'] = 'Buy ' . $product['title'] . ' securely online today at a great price. ' . $product['title'] . ' available today at ' . SITE_NAME.'.';
			}
			else
			{
				$product['metadescription'] = substr(strip_tags($product['description']),0,180);
			}
			
			# Make sure it isn't too big
			$size = 180;
			
			if (strlen($product['metadescription']) > $size)
			{
				$product['metadescription'] = substr($product['metadescription'], 0, $size-3).'...';
			}
		}
		
		if (!isset($product['metakeywords']) || $product['metakeywords'] == '')
		{
			if($product['keyfeatures'])
			{
				$dataforkeywords[] = $product['keyfeatures'];
			}
			if($product['metadescription'])
			{
				//$dataforkeywords[] = $product['metadescription'];
			}
			if($product['title'])
			{
				$dataforkeywords[] = $product['title'];
				$dataforkeywords[] = $product['title'];
				$dataforkeywords[] = $product['title'];
				$dataforkeywords[] = $product['title'];
				$dataforkeywords[] = $product['title'];
			}
			$dataforkeywords = array_merge(buildCategoryKeywords($product_id), buildAttributeKeywords($product_id), $dataforkeywords);
      		$product['metakeywords'] = createKeywordString($dataforkeywords);
		}
		
		
		# Salesrank
		$product['salesrank'] = ($product['salesrank'] == 0 ? 99999999999 : $product['salesrank']);
		
		return ($product);
	}
	
	
	function buildCategoryKeywords($product_id)
	{
		if($product_id)
		{
			$category_paths = getCategoryPathsByProduct($product_id);
			$ending_keywords = array();
			foreach($category_paths as $category_path)
			{
				foreach($category_path as $category_id)
				{
					$category = getCategory($category_id);
					$ending_keywords[] = $category['title'];
					$ending_keywords[] = $category['title'];
					$ending_keywords[] = $category['title'];
				}
			}
			return $ending_keywords;
		}
		else return array();
	}
	
	function buildAttributeKeywords($product_id)
	{
		$attributes = getProductAttributesRaw($product_id);
		$ending_keywords = array();
		# Non-numeric attributes that should be filterd
		$filter_attributes = array("ListPriceCurrencyCode", "ListPriceFormattedPrice", "Warranty", "ProductTypeName", "SKU", "ASIN");
		foreach($attributes as $key => $value)
		{
			if(is_numeric($value) || in_array($key, $filter_attributes) || empty($value) || in_array($value, $ending_keywords))
				continue;
			else
				$ending_keywords[$key] = $value;
		}
		return $ending_keywords;
	}
	
	
	function validateProduct($product)
	{
		if (!isset($product['title']) || $product['title'] == '' )
		{
			setMessage('error', $product['asin'].': Product title is required');
			return false;
		}
		
		if (strlen($product['metadescription']) > 180)
		{
			setMessage('information', $product['asin'].': Meta description should be no longer than 180 characters. (Currently ' . strlen($product['metadescription']) . ' characters)');
		}
		
		return true;
	}
	


	function saveProduct($product_id, $product)
	{
		global $db;
				
		# Generate core info
		$product = generateProductFields($product, $product_id);

		# Validate
		if (!validateProduct($product)) return false;
		
		# Check for a pagename change, insert into legacy
		$oldpagename = $db->value('products', 'pagename', $product_id);
		
		
		if (isset($product['pagename']) && $oldpagename != $product['pagename'])
		{
			setPageRedirect($oldpagename, $product['pagename']);
		}
		
		$db->update('products', $product_id, $product);
		
		return true;
	}
	
	
	
	/*

	function saveProducts($products)
	{
		$product_ids = array();
		foreach ($products as $product)
		{
			$product_ids[$product['asin']] = addProduct($product);
		}
		return ($product_ids);
	}
	
	function addProduct($product)
	{
		global $db;
				
		$product_id = productExists($product['asin']);
		
		
		if ($product_id)
		{
			saveProduct($product_id, $product);
		}
		else
		{
			
			# Generate core info
			$product = generateProductFields($product);
			$product_id = $db->insert('products', $product);
		}
		
		return ($product_id);
	}

	
	
	
	
	
	
	
	
	
	
	
	
	function updateProduct($product)
	{
		global $db;
		
		$parent = $db->cleanValue($product['parent']);
		$parent_id = (isset($product['parent_id']) ? $db->cleanValue($product['parent_id']) : 'parent_id');
		$asin = $db->cleanValue($product['asin']);
		$url = (isset($product['url']) ? $db->cleanValue($product['url']) : '');
		$salesrank = (isset($product['salesrank']) ? $db->cleanValue($product['salesrank']) : '');
		
		$sql = "SELECT id FROM products WHERE asin = '$asin'";
		$rs = $db->query($sql);
		$product_id = $rs->getSingleResult();
		
		if ($url != '' || $salesrank != '')
		{
			$sql = "UPDATE products SET parent = $parent, parent_id = $parent_id, url = '$url', salesrank = '$salesrank', productupdated = NOW() WHERE asin = '$asin'";
			$db->query($sql);
		}
		
		return ($product_id);
	}

	function insertProduct($product)
	{
		global $db;
		
		$parent = $db->cleanValue($product['parent']);
		$parent_id = (isset($product['parent_id']) ? $db->cleanValue($product['parent_id']) : 0);
		$asin = $db->cleanValue($product['asin']);
		$url = (isset($product['url']) ? $db->cleanValue($product['url']) : '');
		$salesrank = (isset($product['salesrank']) ? $db->cleanValue($product['salesrank']) : 0);
		
		$sql = "INSERT INTO products (parent, parent_id, asin, url, salesrank, productupdated) SELECT $parent, $parent_id, '$asin', '$url', '$salesrank', NOW()";
		$rs = $db->query($sql);
		$product_id = $rs->insertID();
		
		return ($product_id);
	}

	*/






	###### Retrieving Products
	function getProduct($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$sql = "SELECT * FROM products WHERE id = '{$product_id}'";
		$rs = $db->query($sql);
		$product = $rs->fetch();
		
		# Price formatted
		$product['priceformatted'] = formatProductPrice($product['price']);
				
		return ($product);
	}
	
	function getProductFeed($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$sql = "SELECT * FROM productfeeds WHERE product_id = '{$product_id}' ORDER BY item_date DESC";
		$rs = $db->query($sql);
		$feeds = $rs->getFullArray();

		foreach($feeds as $key => $feed)
		{
			if(strlen($feed['item_body'] > RSSFEED_CHARACTERS) && RSSFEED_CHARACTERS != 0) $after = '...'; else $after = '';
			$feed['item_body'] = substr(utf8_decode(strip_tags($feed['item_body'])),0, RSSFEED_CHARACTERS);
			$feed['item_body'] .= $after;
			$feeds[$key] = $feed;
		}
		return $feeds;
	}	
	
	function getProductsByCategory($category_id, $page = 1, $sort = 'bestselling', $perpage = PER_PAGE_PRODUCTSBYCATEGORY)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		$page = (int)$page;
		$perpage = (int)$perpage;
		
		$start = ($page-1) * $perpage;
		
		# Sort options
		$orderby = 'salesrank ASC';
		if ($sort == 'price-asc') $orderby = 'price ASC';
		if ($sort == 'price-desc') $orderby = 'price DESC';
		if ($sort == 'reviewscore') $orderby = 'averagescore DESC';
		
		$sql = "SELECT p.* FROM products p, products_categories c WHERE p.id = c.product_id AND p.parent_id = 0 AND c.category_id = '{$category_id}' AND hidden = 0 AND (p.price > 0 OR p.parent = 1 OR p.offersupdated = 0) AND offers > 0 ORDER BY $orderby LIMIT $start, ".$perpage;
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$products = array();
		foreach ($ra as $item)
		{
			$product = $item;
			$product['variationcount'] = getProductVariationCount($item['id']);
			$product['manufacturer'] = getManufacturer($item['manufacturer_id']);
			$product['image'] = getMainProductImage($item['id']);
			$product['priceformatted'] = formatProductPrice($product['price']);
			$products[] = $product;
		}
		return($products);
	}
	
	function countProductsByCategory($category_id, $perpage = PER_PAGE_PRODUCTSBYCATEGORY)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		
		$sql = "SELECT COUNT(*) FROM products p, products_categories c WHERE p.id = c.product_id AND parent_id = 0 AND c.category_id = '{$category_id}' AND hidden = 0 AND (p.price > 0 OR p.parent = 1 OR p.offersupdated = 0) AND offers > 0";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		$pages = ceil($count/$perpage);
		
		$return['pages'] = $pages;
		$return['quantity'] = $count;
		
		
		return($return);
	}
	
	
	
	function getProductsByBrowseNode($browsenode,$searchindex,$page)
	{
		global $amazon;	
		$rawproducts = $amazon->getProductsByBrowseNode($browsenode,$searchindex,$page);
		$rawproducts = $rawproducts['products'];
		$products = array();
		foreach ($rawproducts as $rawproduct)
		{
			$product = array();
			$product = $rawproduct['core'];
			$product['image'] = str_replace('{PARAMETERS}', '.01_SL30_.', $rawproduct['images'][0]);
			$product['title'] = $rawproduct['attributes']['Title'];
			
			if (is_array($rawproduct['editorialreviews']['EditorialReview']) && !isset($rawproduct['editorialreviews']['EditorialReview']['Content']))
			{
				$tempdesc = array_pop($rawproduct['editorialreviews']['EditorialReview']);
				$product['description'] = $tempdesc['Content'];
			}
			else
			{
				$product['description'] = $rawproduct['editorialreviews']['EditorialReview']['Content'];
			}
			
			$product['exists'] = productExists($product['asin']);
			
			$products[] = $product;
		}
		
		return ($products);
	}
	
	function getProductsBySearchTerm($searchindex, $searchterm, $page)
	{
		global $amazon;	
		
		if ($rawproducts = $amazon->getProductsBySearch($searchindex, $searchterm,$page)) {
		
			$rawproducts = $rawproducts['products'];

			$products = array();
			foreach ($rawproducts as $rawproduct)
			{
				$product = array();
				$product = $rawproduct['core'];
				$product['image'] = str_replace('{PARAMETERS}', '.01_SL50_.', $rawproduct['images'][0]);
				$product['title'] = $rawproduct['attributes']['Title'];
				
				if (is_array($rawproduct['editorialreviews']['EditorialReview']) && !isset($rawproduct['editorialreviews']['EditorialReview']['Content']))
				{
					$tempdesc = array_pop($rawproduct['editorialreviews']['EditorialReview']);
					$product['description'] = $tempdesc['Content'];
				}
				else
				{
					$product['description'] = $rawproduct['editorialreviews']['EditorialReview']['Content'];
				}
				
				$product['exists'] = productExists($product['asin']);
				
				$products[] = $product;
			}
			
			
			return ($products);
		}
	}
	
	
		
	
	###### Deleting
	function deleteProducts($products)
	{
		foreach ($products as $product_id)
		{
			deleteProduct($product_id);
		}
	}
	
	function deleteProduct($product_id)
	{
		global $db;
		
		redirectProductPagename($product_id);
		
		deleteProductVariations($product_id);
		deleteProductImages($product_id);
		deleteProductCategories($product_id);
		deleteProductAttributes($product_id);
		deleteProductOptions($product_id);
		deleteProductReviews($product_id);
		deleteProductPromotions($product_id);
		deleteProductOffers($product_id);
		
		$db->delete('productqueues_products', $product_id, 'product_id');
		
		$db->delete('products', $product_id);
	}
	
	
	function redirectProductPagename($product_id)
	{
		global $db;
		# Redirect the page to the homepage
		$product = $db->get('products', $product_id);
		if (isset($product['pagename']) && $product['pagename'] != '')
		{
			# Find parent category
			$categories = getProductCategories($product_id);
			if ($categories)
				$redirect_to = $categories[0]['pagename'];
			else
				$redirect_to = '';
		
			setPageRedirect($product['pagename'], $redirect_to);
		}
	}
	
	
	# Hiding
	
	function hideProducts($products)
	{
		foreach ($products as $product_id)
		{
			hideProduct($product_id);
		}
	}
	
	function hideProduct($product_id)
	{
		global $db;
		
		redirectProductPagename($product_id);
		
		$product_id = $db->cleanValue($product_id);
		$sql = "UPDATE products SET hidden = 1, pagename = 'hidden-".time()."' WHERE id = '{$product_id}'";
		$db->query($sql);
		
		$sql = "DELETE FROM products_categories WHERE product_id = '{$product_id}'";
		$db->query($sql);
		
		$db->delete('productqueues_products', $product_id, 'product_id');
		
	}
	
	function unhideProduct($product_id)
	{
		global $db;
		
		# Re-generate the pagename
		$product = $db->get('products', $product_id);
		$pagename = getUniquePageName($product['pagename'], array('product_id' => $product_id));
		
		$product_id = $db->cleanValue($product_id);
		$sql = "UPDATE products SET hidden = 0, pagename = '{$pagename}' WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	###### Misc
	function getProductAsin($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "SELECT asin FROM products WHERE product_id = '{$product_id}'";
		$rs = $db->query($sql);
		return ($rs->getSingleResult());
	}
	
	function productExists($asin)
	{
		global $db;
		
		$asin = $db->cleanValue($asin);
		$sql = "SELECT id FROM products WHERE asin = '$asin'";
		$rs = $db->query($sql);
		$id = $rs->getSingleResult();
		
		return ($id);
	}
	
	function setProductUpdated($product_id, $type)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$type = $db->cleanValue($type);
				
		$sql = "UPDATE products SET ".$type."updated = NOW() WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	
	/****** Product Attributes */
	
	
	function getProductAttributeNames()
	{
		global $db;
		
		$sql = "SELECT * FROM productattributenames";
		$rs = $db->query($sql);
		return ($rs->getFullArray());
	}

	function saveProductAttributeName($name)
	{
		global $db;
		
		$name = $db->cleanValue($name);
		
		$sql = "SELECT COUNT(*) FROM productattributenames WHERE originalname = '$name'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		if ($count == 0)
		{
			$displayname = splitStringByCaptialLetter($name);
			$sql = "INSERT INTO productattributenames (originalname, displayname, hidden) SELECT '$name','$displayname', 1";
			$db->query($sql);
		}
	}

	function saveProductAttributes($product_id, $attributes)
	{
		global $db;
		
		$attributenames = array();
		
		if(!is_array($attributes)) return;

		# Add/Update
		foreach ($attributes as $name => $value)
		{
			# Translate the value
			$value = translateProductAttribute($name, $value);
			
			# Save the product attribute name
			saveProductAttributeName($name);
			
			# Save it
			if (productAttributeExists($product_id, $name))
			{
				updateProductAttribute($product_id, $name, $value);
			}
			else
			{
				insertProductAttribute($product_id, $name, $value);
			}
			
			$attributenames[] = $name;
			
		}
		
		# Delete ones that no longer exist
		$attributenames = "'".implode("', '", $attributenames)."'";
		$sql = "DELETE FROM productattributes WHERE product_id = '{$product_id}' AND name NOT IN ($attributenames) AND edited = 0";
		$db->query($sql);

		
		setProductUpdated($product_id, 'attributes');
		
		
		# Save Product Manufacturer
		if (isset($attributes['Manufacturer']))
		{
			addProductToManufacturer($product_id, $attributes['Manufacturer']);
		}
		
		
	}
	
	function updateProductAttribute($product_id, $name, $value)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$name = $db->cleanValue($name);
		$value = $db->cleanValue($value);
		
		$attribute = $db->query("SELECT * FROM productattributes a WHERE product_id = $product_id AND a.name = '$name'");
		$attribute = $attribute->fetch();
		
		if ($attribute['edited'] == 0)
		{
			$sql = "UPDATE productattributes SET value = '$value', amazonvalue = '$value' WHERE product_id = '{$product_id}' AND name = '$name'";
		}
		else
		{
			$sql = "UPDATE productattributes SET amazonvalue = '$value' WHERE product_id = '{$product_id}' AND name = '$name'";
		}
		
		$db->query($sql);
	}
	
	function insertProductAttribute($product_id, $name, $value)
	{
		global $db;

		$product_id = $db->cleanValue($product_id);
		$name = $db->cleanValue($name);
		$value = $db->cleanValue($value);
		
		$sql = "INSERT INTO productattributes (product_id, name, value, amazonvalue) SELECT '$product_id', '$name', '$value', '$value'";
		$db->query($sql);
	}


	function deleteProductAttributes($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM productattributes WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	function productAttributeExists($product_id, $name)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$name = $db->cleanValue($name);
		
		$sql = "SELECT COUNT(*) FROM productattributes WHERE product_id = '{$product_id}' and name = '{$name}'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		if ($count == 1) return true; else return false;
	}
	
	function getProductAttributesRaw($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		
		$sql = "SELECT a.name, n.displayname,a.value FROM productattributes a, productattributenames n WHERE product_id = '{$product_id}' AND a.name = n.originalname";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$attributes = array();
		foreach ($ra as $item)
		{
			$attributes[$item['name']] = $item['value'];
		}
		
		return ($attributes);
	}
	
	function getProductAttributes($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		
		$sql = "SELECT a.name, n.displayname, a.value, n.units, n.valuetype FROM productattributes a, productattributenames n WHERE product_id = '{$product_id}' AND a.name = n.originalname AND n.display = 1 AND n.hidden = 0";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$attributes = array();
		foreach ($ra as $item)
		{
			if ($item['valuetype'] == 'yesorno')
			{
				$attributes[$item['displayname']] = ($item['value'] == 1 ? 'Yes' : 'No');
			}
			else
			{
				$attributes[$item['displayname']] = $item['value'].$item['units'];
			}
		}
		
		return ($attributes);
	}
	
	function translateProductAttribute($name, $value)
	{
		global $db;		
	
		if ($value == '') return '';
	
		if ($name == 'BatteriesIncluded')
		{
			$value = ($value == 1 ? 'Yes' : 'No');
		}
		else if ($name == 'ReleaseDate' || $name == 'OriginalReleaseDate')
		{
			$value = date("jS F Y", strtotime($value));
		}
		else if (strstr($value, 'Units hundredths-inches'))
		{
			$value = (int)str_replace(' Units hundredths-inches', '', $value);
			$value = number_format(($value/100) * 2.54, 2);
		}
		else if (strstr($value, 'Units hundredths-pounds'))
		{
			$value = (int)str_replace(' Units hundredths-pounds', '', $value);
			$value = number_format(($value/100) * 0.45359237, 2);
		}
		
		
		return ($value);
	}

	function getSessionProductFilters()
	{
		global $db;
		
		if (isset($_SESSION['productfilters']) && !empty($_SESSION['productfilters']))
		{
			$currentproductfilters = array();
			foreach($_SESSION['productfilters'] as $key => $item)
			{
				if ($key == 'filterprice')
				{
					$currentproductfilters[$key]['attribute'] = false;

					$currentproductfilters[$key]['name'] = 'Price';
					$pricearray = explode('|', $item);
					if ($pricearray[1] == 0) $currentproductfilters[$key]['filters'] = 'more than '.formatProductPrice($pricearray[0]);
					else $currentproductfilters[$key]['filters'] = formatProductPrice($pricearray[0]).' to '.formatProductPrice($pricearray[1]);
				}
				else if ($key == 'filtermanufacturer')
				{
					$currentproductfilters[$key]['attribute'] = false;
					$currentproductfilters[$key]['name'] = 'Manufacturer';
					
					$manufacturer = $db->get('manufacturers', $item);
					
					$currentproductfilters[$key]['filters'] = $manufacturer['linktitle'];
				}
				else
				{
					$currentproductfilters[$key]['attribute'] = $db->get('productattributenames', $key, 'originalname');
					$currentproductfilters[$key]['filters'] = $item;
				}
			}
		}
		else
		{
			$currentproductfilters = false;
		}
		
		return ($currentproductfilters);
	}
	
	function getProductFilterSQL()
	{
		global $db;
		
		if (empty($_SESSION['productfilters'])) return array('1 = 1');
		
		$sql = array();
		
		foreach($_SESSION['productfilters'] as $attributename => $values)
		{
			if ($attributename == 'x' || $attributename == 'y') continue;
			if (is_array($values))
			{
				$valuestring = array();
				foreach($values as $value => $check)
				{
					$valuestring[] = mysql_real_escape_string($value);
				}
				
				$valuestring = implode("', '", $valuestring);
			}
			else
			{
				$valuestring = mysql_real_escape_string($values);
			}
			
			if ($attributename == 'filterprice')
			{
				$price = explode('|',$values);
				$min = $price[0];
				$max = $price[1];
				$sql[] = "(p.price >= $min AND p.price <= $max)";
			}
			else if ($attributename == 'filtermanufacturer')
			{
				# Manufacturer Filter
				$sql[] = "p.manufacturer_id IN($values)";
			}
			else
			{
				$valueparts = explode(" to ", $valuestring);
				$sqlstr = "p.id IN (SELECT x.product_id FROM productattributes x WHERE x.name = '$attributename' AND x.value IN ('$valuestring')";
				if(count($valueparts) == 2 && is_numeric($valueparts[0]) && is_numeric($valueparts[1])) $sqlstr .= " OR (x.value >= {$valueparts[0]} AND x.value < {$valueparts[1]})";
				$sqlstr .= ")";
				$sql[] = $sqlstr;
			}
		}
		
		
		return (!empty($sql))? $sql : array('1 = 1');
	}
		
	function getFilterOptions($sidebar = false)
	{
		global $db;
	
		$filters = array();
		# Price etc. filters
		
		# Sort out price ranges
		$prices = $db->query("SELECT MAX(price) as maxprice, MIN(price) as minprice FROM products WHERE enabled = 1 AND hidden = 0 AND price <> 0 ")->fetch();
		

		
		$priceintervals = FILTER_PRICEINTERVALS;
		
		$start = 0;
		$end = FILTER_PRICEINTERVALS; 
		$filterprices = array();
	
		$i = 0;
		while ((int)$end <= (int)$prices['maxprice'] && (int)$end <= (int)FILTER_PRICEMAXIMUM)
		{
			$i++;
						
			$display = formatProductPrice($start) . ' to ' . formatProductPrice($end);
			$filterprices[] = array('start' => $start, 'end' => $end, 'displayvalue' => $display);
			$start	= $start + $priceintervals;
			if ($i % FILTER_PRICEINTERVALSDOUBLE == 0) $priceintervals = $priceintervals * 2;
			$end	= $end + $priceintervals;
			
			
		}

		$display = formatProductPrice($start).'+';
		$filterprices[] = array('start' => $start, 'end' => $prices['maxprice'], 'displayvalue' => $display);	
		$filters['filterprices'] = $filterprices;

		
		# Get manufacturers for filter
		$filters['filtermanufacturer'] = $db->query("SELECT * FROM manufacturers WHERE display_filter = 1 ORDER BY name")->getFullArray();
	
		$attributes = $db->query("SELECT * FROM productattributenames WHERE hidden = 0 AND metabrowse = 1".($sidebar ? ' AND sidebarfilter = 1' : ''))->getFullArray();
	
		foreach ($attributes as $key => $item)
		{
			$name = $item['originalname'];
			
			if ($item['units'] != '') $orderby = 'ORDER BY CONVERT(value, SIGNED)';
			else $orderby = 'ORDER BY value';
			
			$filters[$key] = $item;
			$filters[$key]['values'] = $db->query("SELECT value, COUNT(*) as quantity FROM productattributes WHERE name = '$name' AND value <> '' GROUP BY value $orderby LIMIT 100")->getFullArray();

			if($item['valuetype'] == 'dropdowngrouped')
			{
				$item['valuetype'] = 'dropdown';
				$min = $filters[$key]['values'][0]['value'];
				$max = $min;
				foreach($filters[$key]['values'] as $key => $value)
				{
					if($value['value'] > $max) $max = $value['value'];
					if($value['value'] < $min) $min = $value['value'];
				}
				
				$divisions = 5; // In the future this could be a parameter if needed
				$dividelength = floor(($max - $max) / $divisions);
				$values = array();
				for($i = $min; $i += $dividelength; $i < $max)
				{
					$values = "{$i} to ".($i + $dividelength);
				}
				$filters[$key]['values'] = $values;
			}
			
			if (!$filters[$key]['values'] || sizeof($filters[$key]['values']) == 1) unset($filters[$key]);
		}
		
		
		
		
		return $filters;
	}
	
	
	/****** Product Images */

	function saveProductImages($product_id, $images)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		if (!$images)
		{
			# Delete existing
			$sql = "DELETE FROM productimages WHERE product_id = '{$product_id}'";
			$db->query($sql);
			return;
		}
		
		# Add Them
		$i = 1;
		foreach ($images as $baseurl)
		{
			insertProductImage($product_id, $baseurl, $i);
			$i++;
		}
		
		
		/*
			# To preserve the sort order, we are not getting fancy
		
		if (!$images)
		{
			$sql = "DELETE FROM productimages WHERE product_id = $product_id";
			$db->query($sql);
			setProductUpdated($product_id, 'images');
			return;
		}
		
		$imageurls = array();
		
		# Add/Update
		foreach ($images as $baseurl)
		{
			if (!productImageExists($product_id, $baseurl))
			{
				insertProductImage($product_id, $baseurl);
			}
			$imageurls[] = $baseurl;
		}
		
		# Delete old ones
		$imageurls = "'".implode("', '", $imageurls)."'";
		$sql = "DELETE FROM productimages WHERE product_id = $product_id AND baseurl NOT IN ($imageurls)";
		$db->query($sql);
		*/
		
		
		setProductUpdated($product_id, 'images');
	}
	
	function insertProductImage($product_id, $baseurl, $sort)
	{
		global $db;

		$product_id = $db->cleanValue($product_id);
		$baseurl = $db->cleanValue($baseurl);
		
		$exists = $db->query("SELECT * FROM productimages WHERE product_id = '{$product_id}' AND baseurl = '$baseurl'")->fetch();
		
		if (!$exists)
		{
			$sql = "INSERT INTO productimages (product_id, source, baseurl, sortorder) SELECT '$product_id', 'amazon', '$baseurl', $sort";
			$db->query($sql);
		}
	}

	function productImageExists($product_id, $imageurl)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$imageurl = $db->cleanValue($imageurl);
		
		$sql = "SELECT COUNT(*) FROM productimages WHERE product_id = '{$product_id}' and baseurl = '$imageurl'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		if ($count == 1) return true; else return false;
	}

	function getMainProductImage($product_id, $size = '110', $default = true)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$sql = "SELECT baseurl FROM productimages WHERE product_id = '{$product_id}' ORDER BY sortorder ASC LIMIT 1";
		$rs = $db->query($sql);
		$image = $rs->getSingleResult();
		
		if (!$image)
		{
			if (!$default) return false;
			$image = '/i/?src='.RELATIVE_URL_IMAGES.'product_noimage.gif&w='.$size.'&h='.$size;
		}
		else
		{
			$parameters = '.01_SL'.$size.'_.';
			$image = str_replace('{PARAMETERS}',$parameters, $image);
		}
		
		return ($image);
	}
	
	function getAllProductImages($product_id, $sizemain, $sizerest = false)
	{
		global $db;
		if (!$sizerest) $sizerest = $sizemain;
		
		$product_id = $db->cleanValue($product_id);
		$sql = "SELECT * FROM productimages WHERE product_id = '{$product_id}' ORDER BY sortorder ASC";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$images = array();
		if (!$ra)
		{

			$images[0]['resized'] = '/i/?src='.RELATIVE_URL_IMAGES.'product_noimage.gif&w='.$sizemain.'&h='.$sizemain;
			$images[0]['full'] = '/i/?src='.RELATIVE_URL_IMAGES.'product_noimage.gif&w=500&h=500';
			$images[0]['noimage'] = true;
		}
		else
		{
			$i = 0;
			foreach ($ra as $item)
			{
				if ($i == 0) $size = $sizemain;
				else $size = $sizerest;
				
				$images[$i]['resized'] = '/img/image_'.$item['id'].'_'.$size.'.jpg';
				$images[$i]['full'] = '/img/image_'.$item['id'].'_500.jpg';
				$images[$i]['noimage'] = false;
				
				$i++;
			}
		}
		
		return ($images);
	}

	function deleteProductImages($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM productimages WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	function getTopProductImageByCategory($category_id, $size = '110')
	{
		global $db;
		
		# First get a list of subcategories...
		$children = getCategoryChildrenIDs($category_id);
		
		if (empty($children))
		{
			$category_ids = array($category_id);
		}
		else
		{
			$category_ids = $children;
			$category_ids[] = $category_id;
		}
				
		$sql = "SELECT p.id FROM products p, products_categories c WHERE p.id = c.product_id AND c.category_id IN (".implode(', ', $category_ids).") AND p.hidden = 0 AND p.salesrank > 0 AND (p.price > 0 OR p.parent = 1) ORDER BY p.salesrank ASC LIMIT 1";
		
		$rs = $db->query($sql);
		$product_id = $rs->getSingleResult();
		
		if (!$product_id)
		{
			$sql = "SELECT p.id FROM products p, products_categories c WHERE p.id = c.product_id AND c.category_id IN (".implode(', ', $category_ids).") AND p.hidden = 0 AND (p.price > 0 OR p.parent = 1) ORDER BY p.salesrank ASC LIMIT 1";
		
			$rs = $db->query($sql);
			$product_id = $rs->getSingleResult();
		}
	
		if (!$product_id) $product_id = 0;
		$image = $db->get('productimages', $product_id, 'product_id', 'sortorder');
		return ($image);
	}

	/****** Products Offers */
	
	
	


	function getProductOffer($product_id)
	{
		global $db, $amazon;
		
		$product_id = $db->cleanValue($product_id);
		
		#Try to get the Amazon offer first
		$attributes = getProductAttributesRaw($product_id);
		$downloadable = (strpos($attributes['ProductTypeName'], "DOWNLOADABLE") !== false)? true : false;

		$offer = $db->query("SELECT * FROM productoffers WHERE product_id = '{$product_id}' AND merchantid = '".$amazon->amazonmerchantid."'")->fetch();

		
		
		# Look for an offer with price available to show
		if (!$offer)
		{		
			$sql = "SELECT * FROM productoffers
				WHERE product_id = $product_id AND price > 0
				ORDER BY price ASC LIMIT 1";
			$rs = $db->query($sql);
			
			$offer = $rs->fetch();
		}
		
		# Look for any offers (including those with a hidden price)
		if (!$offer)
		{		
			$sql = "SELECT * FROM productoffers
				WHERE product_id = $product_id
				ORDER BY price ASC LIMIT 1";
			$rs = $db->query($sql);
			
			$offer = $rs->fetch();
		}
		
		
		# Look for the list price
		if (!$offer || $downloadable)
		{
			$sql = "SELECT value FROM productattributes
				WHERE product_id = '{$product_id}'
				AND name = 'ListPriceAmount'";
			$listpriceraw = $db->query($sql)->getSingleResult();
			
			if($listpriceraw)
			{
				$offer = array(
					'listpriceraw' => $listpriceraw,
					'listprice' => formatProductPrice($listpriceraw)
				);
				return $offer;
			} elseif($downloadable && $offer)
			{
				$offer = array(
					'listpriceraw' => $offer['price'],
					'listprice' => formatProductPrice($offer['price'])
				);
				return $offer;
			}
		}
		
		if (!$offer) return false;
		
		
		$offer['priceraw'] = $offer['price'];
		$offer['priceformatted'] = formatProductPrice($offer['price']);
		$offer['originalpriceformatted'] = formatProductPrice($offer['price']+$offer['amountsaved']);
		$offer['isreduced'] = ($offer['price'] < ($offer['price'] + $offer['amountsaved']) ? true : false );
		
		if ($offer['availabilitytext'] == '') $offer['availabilitytext'] = 'In Stock.';
		
		return ($offer);
	}

	function formatProductPrice($price, $showdecimalsifnone = true)
	{
		global $amazon;
		
		if (!is_numeric($price) && $price != '') return $price;
		
		# Is there a currency conversion rate?
		if (CURRENCYCONVERSION_STATUS != 0)
		{
			$price = $price * CURRENCYCONVERSION_RATE;
			$symbol = CURRENCYCONVERSION_SYMBOL;
		}
		elseif(CURRENCYCONVERSION_SYMBOL != '')
		{
			$symbol = CURRENCYCONVERSION_SYMBOL;
		}
		else
		{
			if ($amazon->locale == 'USA')
			{
				$symbol = '&#36;';
			}
			else if ($amazon->locale == 'UK')
			{
				$symbol = '&pound;';
			}
			else if ($amazon->locale == 'DE')
			{
				$symbol = '&euro;';
			}
			else if ($amazon->locale == 'IT')
			{
				$symbol = '&euro;';
			}
			else if ($amazon->locale == 'FR')
			{
				$symbol = '&euro;';
			}
			else if ($amazon->locale == 'ES')
			{
				$symbol = '&euro;';
			}
			else if ($amazon->locale == 'CA')
			{
				$symbol = '&#36;';
			}
			else if ($amazon->locale == 'JP')
			{
				$symbol = '&yen;';
			}
			else if ($amazon->locale == 'CH')
			{
				$symbol = '&yen;';
			}
			else
			{
				$symbol = '';
			}
		
		}

		$decimals = ($price % 100 == 0 && !$showdecimalsifnone ? 0 : CURRENCYCONVERSION_DECIMALS);
		
		$return = number_format($price / 100,$decimals);
		
		if (CURRENCYCONVERSION_SYMBOLPOSITION == '1')
		{
			$return = $return.$symbol;
		}
		else
		{
			$return = $symbol.$return;
		}
		
		return ($return);
	}
	
	function saveProductOffers($product_id, $offers)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		# No point in updating offers as they change so often, as does the *important* offer listing id
		$sql = "DELETE FROM productoffers WHERE product_id = '{$product_id}'";
		$db->query($sql);
				
		$merchants = array();
		
		# Loop through and add each one
		if (!$offers) $offers = array();
		foreach ($offers as $offer)
		{
		
			$condition = $db->cleanValue($offer['condition']);
			$offerlistingid = $db->cleanValue($offer['offerlistingid']);
			$price = $db->cleanValue($offer['price']);
			$amountsaved = $db->cleanValue($offer['amountsaved']);
			$percentagesaved = $db->cleanValue($offer['percentagesaved']);
			$availabilitytext = $db->cleanValue($offer['availabilitytext']);
			$availabilitytype = $db->cleanValue($offer['availabilitytype']);
			$availabilitymin = $db->cleanValue($offer['availabilitymin']);
			$availabilitymax = $db->cleanValue($offer['availabilitymax']);
			$quantity = $db->cleanValue($offer['quantity']);
			$supersavershipping = $db->cleanValue($offer['supersavershipping']);
			$ispreorder = $db->cleanValue($offer['ispreorder']);
			$merchantid = $db->cleanValue($offer['merchantid']);
			
			$sql = "INSERT INTO productoffers (product_id, productcondition, offerlistingid, price, amountsaved, percentagesaved, availabilitytext, availabilitytype, availabilitymin, availabilitymax, quantity, supersavershipping, ispreorder, merchantid)
					SELECT	$product_id,
							'$condition',
							'$offerlistingid',
							'$price',
							'$amountsaved',
							'$percentagesaved',
							'$availabilitytext',
							'$availabilitytype',
							'$availabilitymin',
							'$availabilitymax',
							$quantity,
							'$supersavershipping',
							'$ispreorder',
							'$merchantid'		
			";
			
			$db->query($sql);
						
			$merchants[$offer['merchant']['merchantid']] = $offer['merchant'];
		}
		
		
		# Update the price on the core product record - for speed in sort orders
		$offer = getProductOffer($product_id);
		if ($offer && $offer['priceraw'] != 0)
		{
			$sql = "UPDATE products SET price = ".$offer['priceraw']." WHERE id = $product_id";
			$db->query($sql);
		}

		
		# Update the product with the number of offers
		$db->update('products', $product_id, array('offers' => sizeof($offers)));
		
		#Save the merchants
		saveMerchants($merchants);
		
		setProductUpdated($product_id, 'offers');
	}
	
	
	function resetProductVariationOffers($parent_id)
	{
		global $db;
		
		$parent_id = $db->cleanValue($parent_id);
		
		# Delete all offers relating to the parent_id
		$db->query("DELETE o FROM productoffers o, products p WHERE o.product_id = p.id AND p.parent_id = '{$parent_id}'");
		$db->query("DELETE FROM productoffers WHERE product_id = '{$parent_id}'");
		
		# Reset the offer count and price field on core product for each variation
		$db->query("UPDATE products SET price = 0, offers = 0 WHERE parent_id = '{$parent_id}' OR id = '{$parent_id}'");
		
		setProductUpdated($parent_id, 'offers');
	}
	
	function updateProductVariationOfferStats($parent_id)
	{
		global $db;
		

		# Update all the variation product records to store the minimum price and total offers
		$rs = $db->query("SELECT id FROM products WHERE parent_id = $parent_id");
		$ra = $rs->getFullArray();
		foreach ($ra as $product)
		{
			$offers_rs = $db->query("SELECT COUNT(*) as offers, MIN(price) as minprice FROM productoffers WHERE product_id = '{$product['id']}'");
			$offers = $offers_rs->fetch();

			$db->query("UPDATE products SET price = '{$offers['minprice']}', offers = '{$offers['offers']}', offersupdated = NOW() WHERE id = '{$product['id']}'");
		}
	
		$rs = $db->query("SELECT IFNULL(SUM(p.offers), 0) as totaloffers, IFNULL(MIN(p.price), 0) as minprice FROM products p WHERE parent_id = '{$parent_id}' AND p.price <> 0");
		$results = $rs->fetch();
		
		# Update the parent product to show total offers and cheapest price
		$db->query("UPDATE products SET price = '{$results['minprice']}', offers = '{$results['totaloffers']}', offersupdated = NOW() WHERE id = '{$parent_id}'");
	}
	
	function saveProductVariationOffer($offer)
	{
		global $db;
		
		$merchants = array();
		
		# Loop through and add each one	
		$asin = $db->cleanValue($offer['asin']);
		$condition = $db->cleanValue($offer['condition']);
		$offerlistingid = $db->cleanValue($offer['offerlistingid']);
		$price = $db->cleanValue($offer['price']);
		$amountsaved = $db->cleanValue($offer['amountsaved']);
		$percentagesaved = $db->cleanValue($offer['percentagesaved']);
		$availabilitytext = $db->cleanValue($offer['availabilitytext']);
		$availabilitytype = $db->cleanValue($offer['availabilitytype']);
		$availabilitymin = $db->cleanValue($offer['availabilitymin']);
		$availabilitymax = $db->cleanValue($offer['availabilitymax']);
		$quantity = $db->cleanValue($offer['quantity']);
		$supersavershipping = $db->cleanValue($offer['supersavershipping']);
		$merchantid = $db->cleanValue($offer['merchantid']);
		
		$sql = "INSERT INTO productoffers (product_id, productcondition, offerlistingid, price, amountsaved, percentagesaved, availabilitytext, availabilitytype, availabilitymin, availabilitymax, quantity, supersavershipping, merchantid)
				SELECT	p.id,
						'$condition',
						'$offerlistingid',
						'$price',
						'$amountsaved',
						'$percentagesaved',
						'$availabilitytext',
						'$availabilitytype',
						'$availabilitymin',
						'$availabilitymax',
						$quantity,
						'$supersavershipping',
						'$merchantid'
				FROM	products	p
				WHERE	p.asin = '$asin'
		";
		
		$db->query($sql);
		
		$merchants[$offer['merchant']['merchantid']] = $offer['merchant'];

			
		# Save the merchants
		saveMerchants($merchants);
	}
	
	
	
	
	
	function deleteProductOffers($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM productoffers WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	
	/****** Produc Options */
	
	
	function getProductOptionNames()
	{
		global $db;
		$sql = "SELECT * FROM productoptionnames ORDER BY originalname DESC";
		$rs = $db->query($sql);
		return($rs->getFullArray());
	}
	

	function saveProductOptions($product_id, $options)
	{
		global $db;
		
		$optionnames = array();
		
		# Add/Update
		if (is_array($options))
		{
			foreach ($options as $name => $value)
			{
				if (productOptionExists($product_id, $name))
				{
					updateProductOption($product_id, $name, $value);
				}
				else
				{
					insertProductOption($product_id, $name, $value);
				}
				
				# Insert Option Name
				saveProductOptionName($name);
				
				$optionnames[] = $name;
			}
			
			# Delete old ones
			$optionnames = "'".implode("', '", $optionnames)."'";
			$sql = "DELETE FROM productoptions WHERE product_id = '{$product_id}' AND name NOT IN ($optionnames)";
			$db->query($sql);
		}
		
		setProductUpdated($product_id, 'options');
	}
	
	function saveProductOptionName($name)
	{
		global $db;
		$name = $db->cleanValue($name);
		
		$sql = "SELECT COUNT(*) FROM productoptionnames WHERE originalname = '$name'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		if ($count == 0)
		{
			$displayname = splitStringByCaptialLetter($name);
			$sql = "INSERT INTO productoptionnames (originalname, displayname) SELECT '$name', '$displayname'";
			$db->query($sql);
		}
	}
	
	function updateProductOption($product_id, $name, $value)
	{
		global $db;

		$product_id = $db->cleanValue($product_id);
		$name = $db->cleanValue($name);
		$value = $db->cleanValue($value);
		
		$sql = "UPDATE productoptions SET value = '$value' WHERE product_id = '{$product_id}' AND name = '$name'";
		$db->query($sql);
	}
	
	function insertProductOption($product_id, $name, $value)
	{
		global $db;

		$product_id = $db->cleanValue($product_id);
		$name = $db->cleanValue($name);
		$value = $db->cleanValue($value);
		
		$sql = "INSERT INTO productoptions (product_id, name, value) SELECT '{$product_id}', '$name', '$value'";
		$db->query($sql);
	}

	function productOptionExists($product_id, $name)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$name = $db->cleanValue($name);
		
		$sql = "SELECT COUNT(*) FROM productoptions WHERE product_id = '{$product_id}' and name = '$name'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		if ($count == 1) return true; else return false;
	}


	function deleteProductOptions($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM productoptions WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	function getParentProductOptions($product_id)

	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		
		$sql = "SELECT id FROM products WHERE parent_id = '{$product_id}'";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		$options = array();
		foreach	($ra as $item)
		{
			$thisid = $item['id'];
			$sql = "SELECT n.displayname, o.value FROM productoptions o, productoptionnames n WHERE o.name = n.originalname AND o.product_id = $thisid ORDER BY n.displayname DESC, o.value DESC";
			$rs = $db->query($sql);
			$rs = $rs->getFullArray();
			$tempoptions = array();
			foreach ($rs as $item)

			{
				$tempoptions[$item['displayname']] = $item['value'];
			}
			
			$options[$thisid]['options'] = $tempoptions;
			
			$sql = "SELECT * FROM productoffers WHERE product_id = '{$thisid}' ORDER BY price ASC LIMIT 1";
			$rs = $db->query($sql);
			$options[$thisid]['offer'] = $rs->fetch();
			$options[$thisid]['offer']['priceformatted'] = formatProductPrice($options[$thisid]['offer']['price']);
			$options[$thisid]['offer']['originalpriceformatted'] = formatProductPrice($options[$thisid]['offer']['price'] + $options[$thisid]['offer']['amountsaved']);
			$options[$thisid]['offer']['savingformatted'] = formatProductPrice($options[$thisid]['offer']['amountsaved']);
		}
		
		return($options);
	}
	
	function getProductOptions($product_id)
	{
		global $db;
		
		$sql = "SELECT n.displayname, o.value FROM productoptions o, productoptionnames n WHERE o.name = n.originalname AND o.product_id = '{$product_id}' ORDER BY n.displayname DESC, o.value DESC";
		$rs = $db->query($sql);
		$rs = $rs->getFullArray();
		$tempoptions = array();
		foreach ($rs as $item)
		{
			$tempoptions[$item['displayname']] = $item['value'];
		}
		
		return ($tempoptions);
	}
	
	function getProductOptionValues($product_id, $optionname)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$optionname = $db->cleanValue($optionname);
		
		$sql = "SELECT	o.value
				FROM	products			p,
						productoptions		o
				WHERE	p.parent_id		= '{$product_id}'
				  AND	p.id			= o.product_id
				  AND	o.name			= '{$optionname}'
				GROUP BY o.value
				ORDER BY o.value ASC";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$optionnames = array();
		foreach ($ra as $item)
		{
			$optionnames[] = $item['value'];
		}
		
		return ($optionnames);
	}
	
	
	function getParentProductOptionSummary($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		
		$sql = "SELECT	o.product_id, MIN(o.price) as price
				FROM	products		p,
						productoffers	o
				WHERE	p.id		= o.product_id
				  AND	p.parent_id	= '{$product_id}'
				GROUP BY o.product_id";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		
		$return['minprice'] = false;
		$return['maxprice'] = false;
		foreach ($ra as $item)
		{
			if (!$return['minprice'] || $item['price'] < $return['minprice']) $return['minprice'] = $item['price'];
			if (!$return['maxprice'] || $item['price'] > $return['maxprice']) $return['maxprice'] = $item['price'];
		}
		
		#$return['minprice'] = formatProductPrice($return['minprice']);
		#$return['maxprice'] = formatProductPrice($return['maxprice']);
		
		
		$sql = "SELECT	n.displayname, o.name
				FROM	productoptionnames	n,
						productoptions		o,
						products			p
				WHERE	n.originalname	= o.name
				  AND	o.product_id	= p.id
				  AND	p.parent_id		= '{$product_id}'
				GROUP BY n.displayname
				ORDER BY n.displayname DESC";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		$options = array();
		foreach ($ra as $item)
		{
			$options[] = $item;
		}
		
		$return['options'] = $options;


		$sql = "SELECT MAX(amountsaved) AS maxsaving, MIN(amountsaved) AS minsaving FROM productoffers o, products p WHERE o.product_id = p.id AND p.parent_id = $product_id";
		$rs = $db->query($sql);
		$ra = $rs->fetch();
		$return['maxsaving'] = $ra['maxsaving'];
		$return['minsaving'] = $ra['minsaving'];
		
		return($return);
	}
	
	
	
	
	/******** Product Promotions */
	
	function deleteProductPromotions($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM productpromotions WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	
	/******** Product Reviews */
	
	
	function deleteProductReviews($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM productreviews WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	function productReviewExists($product_id, $review)
	{
		global $db;
		
		$customerid = $review['customerid'];
		if ($customerid == '') return false;
		
		$sql = "SELECT COUNT(*) FROM productreviews WHERE product_id = '{$product_id}' AND customerid = '$customerid'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		if ($count == 1) return true;
		else return false;
		
	}
	
	function saveProductReviews($product_id, $reviews)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		# First delete any reviews with a blank customer id, because we can't match to update it
		$sql = "DELETE FROM productreviews WHERE product_id = '{$product_id}' AND customerid = ''";
		$db->query($sql);
	
		
		if (is_array($reviews['reviews']) && sizeof($reviews['reviews']) != 0)
		{
			foreach ($reviews['reviews'] as $review)
			{
				if (productReviewExists($product_id, $review))
				{
					updateProductReview($product_id, $review);
				}
				else
				{
					insertProductReview($product_id, $review);
				}
			}
		}
		
		# Update the average score on the core product, for sorting and displaying easily
		$averagescore = isset($reviews['reviewinfo']['averagescore']) ? $reviews['reviewinfo']['averagescore'] : 0;
		$totalreviews = isset($reviews['reviewinfo']['totalreviews']) ? $reviews['reviewinfo']['totalreviews'] : 0;
		$iframeurl = $reviews['reviewinfo']['iframeurl'];
		$sql = "UPDATE products SET averagescore = '{$averagescore}', totalreviews = '{$totalreviews}', reviewiframelink = '$iframeurl' WHERE id = '{$product_id}'";
		$db->query($sql);
		
		setProductUpdated($product_id, 'reviews');
	}
	
	function updateProductReview($product_id, $review)
	{
		global $db;

		$product_id = $db->cleanValue($product_id);
		$rating = $db->cleanValue($review['rating']);
		$helpfulvotes = $db->cleanValue($review['helpfulvotes']);
		$totalvotes = $db->cleanValue($review['totalvotes']);
		$customerid = $db->cleanValue($review['customerid']);
		$reviewername = $db->cleanValue($review['reviewername']);
		$reviewernickname = $db->cleanValue($review['reviewernickname']);
		$reviewerlocation = $db->cleanValue($review['reviewerlocation']);
		$date = $db->cleanValue($review['date']);
		$title = $db->cleanValue($review['title']);
		$review = $db->cleanValue($review['review']);
		
		$sql = "UPDATE	productreviews
				SET		product_id = '$product_id',
						rating = '$rating',
						helpfulvotes = '$helpfulvotes',
						totalvotes = '$totalvotes',
						customerid = '$customerid',
						reviewername = '$reviewername',
						reviewernickname = '$reviewernickname',
						reviewerlocation = '$reviewerlocation',
						date = '$date',
						title = '$title',
						review = '$review'
				WHERE	product_id = $product_id
				  AND	customerid = '$customerid'";
				  
		$db->query($sql);
	}
	
	function insertProductReview($product_id, $review)
	{
		global $db;

		$product_id = $db->cleanValue($product_id);
		$rating = $db->cleanValue($review['rating']);
		$helpfulvotes = $db->cleanValue($review['helpfulvotes']);
		$totalvotes = $db->cleanValue($review['totalvotes']);
		$customerid = $db->cleanValue($review['customerid']);
		$reviewername = $db->cleanValue($review['reviewername']);
		$reviewernickname = $db->cleanValue($review['reviewernickname']);
		$reviewerlocation = $db->cleanValue($review['reviewerlocation']);
		$date = $db->cleanValue($review['date']);
		$title = $db->cleanValue($review['title']);
		$review = $db->cleanValue($review['review']);
		
		$sql = "INSERT INTO productreviews (product_id, rating, helpfulvotes, totalvotes, customerid, reviewername, reviewernickname, reviewerlocation, date, title, review)
				SELECT	'$product_id',
						'$rating',
						'$helpfulvotes',
						'$totalvotes',
						'$customerid',
						'$reviewername',
						'$reviewernickname',
						'$reviewerlocation',
						'$date',
						'$title',
						'$review'";
				  
		$db->query($sql);
	} 


	function getProductReviews($product_id)
	{
		global $db;
		
		$sql = "SELECT * FROM productreviews WHERE product_id = '{$product_id}' ORDER BY date DESC";
		$rs = $db->query($sql);

		$ra = $rs->getFullArray();
		
		return ($ra);
	}
	
	function getMostRecentReview($product_id)
	{
		global $db;
		
		$sql = "SELECT * FROM productreviews WHERE product_id = '{$product_id}' ORDER BY date DESC LIMIT 1";
		$rs = $db->query($sql);
		$ra = $rs->fetch();
		
		return ($ra);
	}


	/****** Product Status */
	
	function disableProducts($products)
	{
		foreach ($products as $product_id)
		{
			disableProduct($product_id);
		}
	}
	
	function disableProduct($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		$sql = "UPDATE products SET enabled = 0 WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	function enableProducts($products)
	{
		foreach ($products as $product_id)
		{
			enableProduct($product_id);
		}
	}
	
	function enableProduct($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		$sql = "UPDATE products SET enabled = 1 WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	function changeProductStatus($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		$sql = "UPDATE products SET enabled = CASE enabled WHEN 1 THEN 0 ELSE 1 END WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	
	/****** Product Updating */
	
		###### Product Updating
	function updateAllProductInformation()
	{
		#echo getTimestamp();
		checkProductOffersToUpdate();
		#echo getTimestamp();
		checkProductsToUpdate();
		#echo getTimestamp();
		checkProductAttributesToUpdate();
		#echo getTimestamp();
		checkProductVariationsToUpdate();
		#echo getTimestamp();
		checkProductImagesToUpdate();
		#echo getTimestamp();
		checkProductReviewsToUpdate();
		#echo getTimestamp();
		checkProductPromotionsToUpdate();
		#echo getTimestamp();
		checkProductFeedsToUpdate();
	}

	
	function checkProductFeedsToUpdate()
	{
		global $db;
		
		$sql = "SELECT *
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, feedupdated, NOW()) > ".CACHE_TIME_PRODUCT_BASIC." OR feedupdated = 0)
				  AND enabled	= 1
				  AND hidden	= 0
				  AND feedurl <> ''
				ORDER BY feedupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		require_once PATH_LIBRARY.'magpierss/rss_fetch.php';
		
		foreach ($ra as $key => $item)
		{
			# Load feed
			$feedurl = $item['feedurl'];
			$feedrss = fetch_rss($feedurl);
			
			foreach($feedrss->items as $feeditem)
			{
				# Check if item is already entered
				$sql = "SELECT COUNT(*)
						FROM productfeeds
						WHERE product_id = '{$item['id']}'
							AND item_url = '{$feeditem['link']}'
						LIMIT 1";
				$rs = $db->query($sql);
				$cnt = $rs->getSingleResult();
				
				if($cnt == 0)
				{
					# Insert feed item
					$item_date = date('Y-m-d H:i:s', strtotime($feeditem['pubdate']));
					$item_url = mysql_real_escape_string($feeditem['link']);
					$item_title = mysql_real_escape_string($feeditem['title']);
					$item_body = mysql_real_escape_string(strip_tags($feeditem['content']['encoded'], '<p><a><br>'));
					$sql = "INSERT INTO productfeeds (product_id, item_date, item_url, item_title, item_body) VALUES ('{$item['id']}', '{$item_date}', '{$item_url}', '{$item_title}', '{$item_body}')";
					$db->query($sql);
				}
			}
			
			# Count items
			$sql = "SELECT COUNT(*)
					FROM productfeeds
					WHERE product_id = '{$item['id']}'";
			$rs = $db->query($sql);
			$cnt = $rs->getSingleResult()-RSSFEED_COUNT;

			if($cnt > 0)
			{
				# Delete any old items
				$sql = "DELETE FROM productfeeds
						WHERE product_id = '{$item['id']}'
						ORDER BY item_date ASC
						LIMIT {$cnt}";
				$db->query($sql);
			}
		
			# Update last updated
			$sql = "UPDATE products SET feedupdated = NOW() WHERE id = '{$item['id']}' LIMIT 1";
			$db->query($sql);
		}
	}
	
	function updateProductFeed($product)
	{
		global $db;
		
		if ((time() - strtotime($product['feedupdated'])) < CACHE_TIME_PRODUCT_BASIC)
		{
			return false;
		}
		
		require_once PATH_LIBRARY.'/magpierss/rss_fetch.php';
		
		# Load feed
		$feedurl = $product['feedurl'];
		if($feedurl)
		{
			$feedrss = fetch_rss($feedurl);
		
			foreach($feedrss->items as $feeditem)
			{
				# Check if item is already entered
				$sql = "SELECT COUNT(*)
						FROM productfeeds
						WHERE product_id = '{$product['id']}'
							AND item_url = '{$feeditem['link']}'
						LIMIT 1";
				$rs = $db->query($sql);
				$cnt = $rs->getSingleResult();
			
				if($cnt == 0)
				{
					# Insert feed item
					$item_date = date('Y-m-d H:i:s', strtotime($feeditem['pubdate']));
					$item_url = mysql_real_escape_string($feeditem['link']);
					$item_title = mysql_real_escape_string($feeditem['title']);
					$item_body = mysql_real_escape_string(strip_tags($feeditem['content']['encoded'], '<p><a><br>'));
					$sql = "INSERT INTO productfeeds (product_id, item_date, item_url, item_title, item_body) VALUES ('{$product['id']}', '{$item_date}', '{$item_url}', '{$item_title}', '{$item_body}')";
					$db->query($sql);
				}
			}
		
			# Count items
			$sql = "SELECT COUNT(*)
					FROM productfeeds
					WHERE product_id = '{$product['id']}'";
			$rs = $db->query($sql);
			$cnt = $rs->getSingleResult()-RSSFEED_COUNT;

			if($cnt > 0)
			{
				# Delete any old items
				$sql = "DELETE FROM productfeeds
						WHERE product_id = '{$product['id']}'
						ORDER BY item_date ASC
						LIMIT {$cnt}";
				$db->query($sql);
			}
	
			# Update last updated
			$sql = "UPDATE products SET feedupdated = NOW() WHERE id = '{$product['id']}' LIMIT 1";
			$db->query($sql);
		} else return false;
	}
	
	function checkProductsToUpdate()
	{
		global $db, $amazon;
		
		$sql = "SELECT *
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, productupdated, NOW()) > ".CACHE_TIME_PRODUCT_BASIC." OR productupdated = '0000-00-00 00:00:00')
				  AND enabled	= 1
				  AND hidden	= 0
				ORDER BY productupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();

		foreach ($ra as $key => $item)
		{
			$product = $amazon->getProductFull($item['asin']);
			
			# Product may be disabled (handled in class) or comms error
			if (!$product) continue;
			
			if (!is_array($product) || $product['core']['asin'] == '')
			{
				# Problem getting the product, but not necessarily disabled
				continue;
			}
			
			
			$product_id = saveAmazonProduct($product);
			
			# Images and Attributes
			saveProductAttributes($item['id'], $product['attributes']);
			saveProductImages($item['id'], $product['images']);
						
		}
	}
	
	function updateProductMain($product)
	{
		#Does it need to be updated
		if ((time() - strtotime($product['productupdated'])) < CACHE_TIME_PRODUCT_BASIC)
		{
			return false;
		}
	
		global $db, $amazon;
		
		$azproduct = $amazon->getProductFull($product['asin']);
			
		# Product may be disabled (handled in class) or comms error
		if (!$azproduct) return;
		
		if (!is_array($azproduct) || $azproduct['core']['asin'] == '')
		{
			# Problem getting the product, but not necessarily disabled
			return;
		}
		
		$product_id = saveAmazonProduct($azproduct);
		
		# Images and Attributes
		saveProductAttributes($product['id'], $azproduct['attributes']);
		saveProductImages($product['id'], $azproduct['images']);
	}
	
	function checkProductAttributesToUpdate()
	{
		global $db, $amazon;
		
		$sql = "SELECT id, asin
				FROM products
				WHERE UNIX_TIMESTAMP() > (UNIX_TIMESTAMP(attributesupdated) + (60 * ".CACHE_TIME_PRODUCT_ATTRIBUTES."))
				  AND enabled	= 1
				  AND hidden	= 0
				ORDER BY attributesupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		foreach ($ra as $key => $item)
		{
			$attributes = $amazon->getProductAttributes($item['asin']);
			if (!$attributes) continue;
			saveProductAttributes($item['id'], $attributes);
		}
	}
	
	function updateProductAttributes($product)
	{
		#Does it need to be updated
		if ((time() - strtotime($product['productupdated'])) < CACHE_TIME_PRODUCT_ATTRIBUTES)
		{
			return false;
		}
	
		global $db, $amazon;
		
		$attributes = $amazon->getProductAttributes($item['asin']);
		if (!$attributes) return;
		saveProductAttributes($item['id'], $attributes);
	} 
	
	
	
	function checkProductVariationsToUpdate()
	{
		global $db;
		
		$sql = "SELECT id, asin
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, variationsupdated, NOW()) > ".CACHE_TIME_PRODUCT_VARIATIONS." OR variationsupdated = '0000-00-00 00:00:00')
				  AND enabled	= 1
				  AND parent	= 1
				  AND hidden	= 0
				ORDER BY variationsupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
				
		foreach ($ra as $key => $item)
		{
			updateProductVariations($item['id'], $item['asin']);
		}
	}
	
	function checkProductOffersToUpdate()
	{
		global $db, $amazon;
		
		# Single Products - i.e. no variations
		$sql = "SELECT id, asin, marketplaceitem, TIMESTAMPDIFF (MINUTE, offersupdated, NOW()) as timesinceupdate
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, offersupdated, NOW()) > ".CACHE_TIME_PRODUCT_OFFERS." OR offersupdated = '0000-00-00 00:00:00')
				  AND enabled	= 1
				  AND parent	= 0
				  AND parent_id = 0
				  AND hidden	= 0
				ORDER BY offersupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
				
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		foreach ($ra as $key => $item)
		{
			$offers = $amazon->getProductOffers($item['asin'], PRODUCT_OFFERS_MERCHANTID, $item['marketplaceitem']);
			if (!is_array($offers)) continue;
			saveProductOffers($item['id'], $offers);
		}
		
		# Variation Products - these are updated in a batch
		$sql = "SELECT x.asin, x.id
				FROM products p, products x
				WHERE p.parent_id = x.id
				  AND (TIMESTAMPDIFF (MINUTE, p.offersupdated, NOW()) > ".CACHE_TIME_PRODUCT_OFFERS." OR p.offersupdated = '0000-00-00 00:00:00')
				  AND x.enabled	= 1
				  AND p.parent	= 0
				  AND x.parent	= 1
				  AND p.hidden	= 0
				  AND x.hidden	= 0
				GROUP BY x.asin, x.id
				ORDER BY x.offersupdated ASC, p.salesrank ASC, p.viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		foreach ($ra as $key => $item)
		{
			resetProductVariationOffers($item['id']);
			$success = $amazon->getProductVariationOffers($item['asin'], 'saveProductVariationOffer');

			if (!$success) continue;
			updateProductVariationOfferStats($item['id']);
		}
	}
	
	function updateProductOffers($product, $bypasscache = false)
	{
		# Does it need updating?
		if (((strtotime($product['offersupdated']) + (CACHE_TIME_PRODUCT_OFFERS * 60)) > time()) && !$bypasscache)
		{
			return false;
		}
		
		global $db, $amazon;
		
		if ($product['parent'] == 1)
		{		
			resetProductVariationOffers($product['id']);
			$amazon->getProductVariationOffers($product['asin'], 'saveProductVariationOffer');
			updateProductVariationOfferStats($product['id']);
		}
		else
		{	
			$offers = $amazon->getProductOffers($product['asin'], PRODUCT_OFFERS_MERCHANTID, $product['marketplaceitem']);		
			saveProductOffers($product['id'], $offers);
		}
	}
	
	function checkProductImagesToUpdate()
	{
		global $db, $amazon;
		
		$sql = "SELECT id, asin
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, imagesupdated, NOW()) > ".CACHE_TIME_PRODUCT_IMAGES." OR imagesupdated = '0000-00-00 00:00:00')
				  AND enabled	= 1
				  AND hidden	= 0
				ORDER BY imagesupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
				
		foreach ($ra as $key => $item)
		{
			$images = $amazon->getProductImages($item['asin']);		
			if (!$images) continue;
			saveProductImages($item['id'], $images);
		}
	}
	
	function updateProductImages($product, $forced = false)
	{
		# Does it need updating?
		if (((strtotime($product['imagesupdated']) + (CACHE_TIME_PRODUCT_OFFERS * 60)) > time()) && !$forced)
		{
			return;
		}
		
		global $db, $amazon;
		
		$images = $amazon->getProductImages($product['asin']);
		if (!$images) return;
		saveProductImages($product['id'], $images);
	}
	
	function checkProductReviewsToUpdate()
	{
		global $db, $amazon;
		
		$sql = "SELECT id, asin
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, reviewsupdated, NOW()) > ".CACHE_TIME_PRODUCT_REVIEWS." OR reviewsupdated = '0000-00-00 00:00:00')
				  AND enabled	= 1
				  AND parent_id = 0
				  AND hidden	= 0
				ORDER BY reviewsupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 5";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
				
		foreach ($ra as $key => $item)
		{
			$reviews = $amazon->getProductReviews($item['asin']);
			saveProductReviews($item['id'], $reviews);
		}
	}
	
	function updateProductReviews($product, $bypasscache = false)
	{
		# Does it need updating?
		if (((strtotime($product['reviewsupdated']) + (CACHE_TIME_PRODUCT_REVIEWS * 60)) > time()) && !$bypasscache)
		{
			return false;
		}
		
		global $amazon;
		
		$reviews = $amazon->getProductReviews($product['asin']);
		saveProductReviews($product['id'], $reviews);
		
		return true;
	}
	
	function checkProductPromotionsToUpdate()
	{
		return true; # Not used for the time being
		
		global $db, $amazon;
		
		$sql = "SELECT id, asin
				FROM products
				WHERE (TIMESTAMPDIFF (MINUTE, promotionsupdated, NOW()) > ".CACHE_TIME_PRODUCT_PROMOTIONS." OR promotionsupdated = '0000-00-00 00:00:00')
				  AND enabled	= 1
				  AND parent_id = 0
				  AND hidden	= 0
				ORDER BY promotionsupdated ASC, salesrank ASC, viewcount DESC
				LIMIT 50";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
				
		foreach ($ra as $key => $item)
		{
			$promotions = $amazon->getProductPromotions($item['asin']);
			if (!$promotions) continue;
			#saveProductPromotions($item['id'], $promotions);
		}
	
	}
	
	
	/******** Product Variations */
	
	

	function deleteProductVariations($product_id)
	{
		$variations = getProductVariations($product_id);
		if (is_array($variations))
		{
			foreach ($variations as $variation)
			{
				deleteProduct($variation['id']);
			}
		}
	}

	function getProductVariationCount($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		$sql = "SELECT COUNT(*) FROM products WHERE parent_id = '{$product_id}'";
		$rs = $db->query($sql);
		return ($rs->getSingleResult());
	}
	
	function checkUpdateProductVariations($product)
	{
		#Does it need to be updated
		if ((time() - strtotime($product['productupdated'])) < CACHE_TIME_PRODUCT_VARIATIONS)
		{
			return false;
		}
		
		updateProductVariations($product['id'], $product['asin']);
	}
	

	function updateProductVariations($product_id, $asin = '')
	{
		global $db, $amazon;
	
		if ($asin == '') getProductAsin($product_id);
		$parent = $db->get('products', $product_id);
		
		$variations = $amazon->getProductVariations($asin);
		
		if ($variations)
		{
			foreach ($variations['items'] as $item)
			{
				$product = array();
				$product['asin'] = trim($item['ASIN']);
				$product['parent'] = 0;
				$product['parent_id'] = $product_id;
				$product['url'] = str_replace($asin, $product['asin'], $parent['url']);
				
				# Check for bad info
				if ($product['asin'] == '') continue;
			

				$variation_product_id = productExists($product['asin']);
			
				if ($variation_product_id)
				{
					$db->update('products', $variation_product_id, $product);
				}
				else
				{
					$variation_product_id = $db->insert('products', $product);
				}

				foreach($item['ItemAttributes'] as $key => $value) $item['ItemAttributes'][$key] = parseForeignLanguageCharacters($value);
			
				saveProductOptions($variation_product_id, $item['ItemAttributes']);
			}

		}
		
		setProductUpdated($product_id, 'variations');
	}
	
	function getProductVariations($parent_id)
	{
		global $db;
		
		$parent_id = $db->cleanValue($parent_id);
		
		$sql = "SELECT * FROM products WHERE parent_id = '{$parent_id}'";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		return ($ra);
	}




	/******** Products to Categories */
	
	function addProductsToCategory($products, $category_id)
	{
		foreach ($products as $product_id)
		{		
			addProductToCategory($product_id, $category_id);
		}
	}
	
	function addProductToCategory($product_id, $category_id)
	{


		global $db;
		
		if (!isProductInCategory($product_id, $category_id))
		{
			$product_id = $db->cleanValue($product_id);
			$category_id = $db->cleanValue($category_id);
			$sql = "INSERT INTO products_categories (product_id, category_id) SELECT '{$product_id}', '{$category_id}'";
			$db->query($sql);
		}
	}
	
	function isProductInCategory($product_id, $category_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		$category_id = $db->cleanValue($category_id);
		
		$sql = "SELECT COUNT(*) FROM products_categories WHERE product_id = '{$product_id}' AND category_id = '{$category_id}'";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		if ($count == 0) return false;
		else return true;		
	}
	
	function isProductsInCategory($category_id, $minimum_amount = 0)
	{
		global $db;
		
		$category_id = $db->cleanValue($category_id);
		
		$sql = "SELECT COUNT(*) FROM products_categories c, products p WHERE c.category_id = '{$category_id}' AND p.hidden = 0 AND (p.price > 0 OR p.parent = 1)";
		$rs = $db->query($sql);
		$count = $rs->getSingleResult();
		
		if ($count <= $minimum_amount) return false;
		else return true;	
	}
	
	
	function removeProductFromCategory($product_id, $category_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$category_id = $db->cleanValue($category_id);
		
		$sql = "DELETE FROM products_categories WHERE product_id = '{$product_id}' AND category_id = '{$category_id}'";
		$db->query($sql);
		
		# Also set an explicit product/category ban. To prevent auto population adding this back in.
		$data['product_id'] = $product_id;
		$data['category_id'] = $category_id;
		$db->insert('products_categories_banned', $data);
		
	}
	
	function deleteProductCategories($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "DELETE FROM products_categories WHERE product_id = '{$product_id}'";
		$db->query($sql);
	}
	
	function getProductCategories($product_id)
	{
		global $db;
		$product_id = $db->cleanValue($product_id);
		$sql = "SELECT category_id FROM products_categories WHERE product_id = '{$product_id}'";
		$rs = $db->query($sql);
		$ra = $rs->getFullArray();
		
		if (!$ra) return false;
		
		$categories = array();
		foreach ($ra as $item)
		{

			$categories[] = getCategory($item['category_id']);
		}
		
		return($categories);
	}
	
	/* Product lock status */
	function unlockProducts($products)
	{
		foreach ($products as $product_id)
		{
			unlockProduct($product_id);
		}
	}
	
	function unlockProduct($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		$sql = "UPDATE products SET edited = 0 WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
	function lockProducts($products)
	{
		foreach ($products as $product_id)
		{
			lockProduct($product_id);
		}
	}
	
	function lockProduct($product_id)
	{
		global $db;
		
		$product_id = $db->cleanValue($product_id);
		
		$sql = "UPDATE products SET edited = 1 WHERE id = '{$product_id}'";
		$db->query($sql);
	}
	
		
?>
