<?php

	/*
		- Gets the configuration file contents (config.php) and parses into the database
		- Doesn't ovverride any existing settings in the database
	*/
	function getTemplateConfig($templatekey)
	{
		global $db;
		
		$configfile = PATH_ROOT.'templates/'.$templatekey.'/config.php';
		
		if (file_exists($configfile))
		{
			#This will give us the $template array with config settings
			require ($configfile);
			if (!$template || !is_array($template)) return false;
			
			
			#Parse the options
			if (isset($template['options']) && is_array(($template['options'])))
			{
				$optionkeys = array();
				foreach ($template['options'] as $key => $option)
				{
					# Check db for existing option
					$rs = $db->query("SELECT * FROM template_options WHERE templateuid = '$templatekey' AND optionuid = '$key'");
					
					if ($rs->rows() == 0)
					{
						#Insert the option
						$insert['templateuid'] = $templatekey;
						$insert['optionuid'] = $key;
						$insert['name'] = $option['name'];
						$insert['description'] = $option['description'];
						$insert['datatype'] = $option['datatype'];
						$insert['editable'] = $option['editable'];
						$insert['default_value'] = $option['default_value'];
						$insert['created'] = 'NOW()';
						$db->insert('template_options', $insert);
					}
					else
					{
						$originaloption = $rs->fetch();
						$update = $option;
						$update['deleted'] = 0;
						$db->update('template_options', $originaloption['id'], $update);
					}
					
					$optionkeys[] = $key;
				}
				
				# Delete options no longer in the config file
				$db->query("UPDATE template_options SET deleted = 1 WHERE templateuid = '$templatekey' AND optionuid NOT IN ('".implode("','", $optionkeys)."')");
			}
			
			# Parse the images
			if (isset($template['images']) && is_array(($template['images'])))
			{
				$imagekeys = array();
				foreach ($template['images'] as $key => $image)
				{
					# Check db for existing image
					$rs = $db->query("SELECT * FROM template_images WHERE templateuid = '$templatekey' AND imageuid = '$key'");
					
					if ($rs->rows() == 0)
					{
						#Insert the image
						$insert = $image;
						$insert['templateuid'] = $templatekey;
						$insert['imageuid'] = $key;
						$insert['created'] = 'NOW()';
						$db->insert('template_images', $insert);
					}
					else
					{
						$originalimage = $rs->fetch();
						$update = $image;
						$update['deleted'] = 0;
						$db->update('template_images', $originalimage['id'], $update);
					}
					
					$imagekeys[] = $key;
				}
				
				# Delete options no longer in the config file
				$db->query("UPDATE template_images SET deleted = 1 WHERE templateuid = '$templatekey' AND imageuid NOT IN ('".implode("','", $imagekeys)."')");
			}
			
			
			# Parse the CSS
			if (isset($template['css']) && is_array(($template['css'])))
			{
				$imagekeys = array();
				foreach ($template['css'] as $key => $css)
				{
					# Check db for existing image
					$rs = $db->query("SELECT * FROM template_css WHERE templateuid = '$templatekey' AND cssuid = '$key'");
					
					if ($rs->rows() == 0)
					{
						#Insert the CSS
						$insert = $css;
						$insert['templateuid'] = $templatekey;
						$insert['cssuid'] = $key;
						$insert['created'] = 'NOW()';
						$db->insert('template_css', $insert);
					}
					else
					{
						$originalcss = $rs->fetch();
						$update = $css;
						$update['deleted'] = 0;
						$db->update('template_css', $originalcss['id'], $update);
					}
					
					$csskeys[] = $key;
				}
				
				# Delete options no longer in the config file
				$db->query("UPDATE template_css SET deleted = 1 WHERE templateuid = '$templatekey' AND cssuid NOT IN ('".implode("','", $csskeys)."')");
			}
			
			
			return true;
		}
		else
		{
			return false;
		}
	}


	function getTemplateOption($optionuid)
	{
		global $db;
	
		# Get the current template
		$templateuid = $db->value('parameters', 'value', 'TEMPLATE_FOLDER', 'parameter');
		
		$option = $db->query("SELECT * FROM template_options WHERE templateuid = '$templateuid' AND optionuid = '$optionuid'")->fetch();
		
		if (!$option) return false;
		
		if ($option['datatype'] == 'products')
		{
			$product_ids = str_replace('|', ',', $option['value']);
			if ($product_ids == '') $product_ids = '0';
			$products = $db->query("SELECT * FROM products WHERE id IN ($product_ids)")->getFullArray();
			
			foreach ($products as $key => $product)
			{
				if ($product['parent'])
				{
					$product['optionsummary'] = getParentProductOptionSummary($product['id']);
					$product['price'] = $product['optionsummary']['minprice'];
				}
					
				$product['variationcount'] = getProductVariationCount($product['id']);
				$product['manufacturer'] = getManufacturer($product['manufacturer_id']);
				$product['image'] = $db->get('productimages', $product['id'], 'product_id', 'sortorder');
				$product['priceformatted'] = formatProductPrice($product['price']);
				$product['reviewcount'] = $db->value('productreviews', 'COUNT(*)', $product['id'], 'product_id');
				$product['offer'] = getProductOffer($product['id']);
				$products[$key] = $product;
			}
			
			return ($products);
		}
		else if ($option['datatype'] == 'pages')
		{
			if ($option['value'] == '') return array();
			
			# Explode the option for sort order
			$page_ids = explode('|', $option['value']);
			
			$pages = array();
			foreach ($page_ids as $sort => $page_id)
			{
				if(is_numeric($page_id)) $pages[$sort] = $db->query('SELECT * FROM pages WHERE id = '.$page_id)->fetch();
			}

			return ($pages);
		}
		else
		{
			return ($option['value'] == '' ? $option['default_value'] : $option['value']);
		}
	}
	
	function getTemplateImage($imageuid, $cssid = '', $cssclass = '', $default = '')
	{
		global $db;
	
		# Get the current template
		$templateuid = $db->value('parameters', 'value', 'TEMPLATE_FOLDER', 'parameter');
		
		$image = $db->query("SELECT * FROM template_images WHERE templateuid = '$templateuid' AND imageuid = '$imageuid'")->fetch();
		
		if (!$image || $image['disabled'] == '1') return '';
		
		$html = '';
		
		$source = $image['src'];
		
		# Check for a default passed into the function.. could be anything (e.g. text)
		if ($source == '' && $default != '')
		{
			return $default;
		}
		
		if ($source == '')
		{
			# Get default
			if ($image['default_src'] == '') return '';
			$default_source = $image['default_src'];
			
			# Check for a language alternative
			$language = getTemplateOption('TEMPLATE_LANGUAGE');
			$languageoverride = false;
			if ($language && $language != 'uk')
			{
				$languageoverride = true;
				$source = str_replace('%TEMPLATEFOLDER%', URL_SITE.'templates/'.$templateuid.'/languages/'.$language, $default_source);
			}
								
			if (!$languageoverride || !file_exists(str_replace(URL_SITE, '', $source)))
			{
				$source = str_replace('%TEMPLATEFOLDER%', URL_SITE.'templates/'.$templateuid, $default_source);
			}
		}
		
		$html .= '<img src="'.$source.'" alt="'.$image['alt'].'" title="'.$image['title'].'"';
		if ($cssid != '') $html .= ' id="'.$cssid.'"';
		if ($cssclass != '') $html .= ' class="'.$cssclass.'"';
		$html .= ' />';
		
		$link = $image['link'];
		
		if ($link == '' && $image['default_link'] !== '')
		{
			if (strstr($image['default_link'], '[PAGE]'))
			{
				$page = $db->get('pages', str_replace('[PAGE]', '', $image['default_link']), 'pagekey');
				$link = URL_SITE.($page['pagename'] != '' ? $page['pagename'].'/' : '');
			}
			else
			{
				$link = $image['default_link'];
			}
		}
		
		if ($link != '')
		{
			$html = '<a href="'.$link.'"'.($image['linknofollow'] ? ' rel="nofollow"' : '').($image['linktarget'] ? ' target="'.$image['linktarget'].'"' : '').'>'.$html.'</a>';
		}
		
		return ($html);
	}
	
	function getTemplateCSS()
	{
		global $db;
	
		# Get the current template
		$templateuid = $db->value('parameters', 'value', 'TEMPLATE_FOLDER', 'parameter');
	
		$return = '<style type="text/css">'."\n";
	
		# Get the CSS options
		$css = $db->query("SELECT * FROM template_css WHERE templateuid = '$templateuid' AND decleration <> ''")->getFullArray();
		foreach ($css as $item)
		{
			$return .= $item['decleration']."\n";
		}
		
		# Get the image CSS
		$imagecss = $db->query("SELECT * FROM template_images WHERE templateuid = '$templateuid' AND type='css' AND cssdecleration <> ''")->getFullArray();
		foreach ($imagecss as $item)
		{
			$return .= $item['cssselector'].' '.$item['cssdecleration']."\n";
		}
	
	
		$return .= '</style>';
	
		return($return);
	}
	
	function initialiseTemplateText()
	{
		global $templatetext;
		
		$language = getTemplateOption('TEMPLATE_LANGUAGE');
		if (empty($language)) $language = "en";
		
		$languagefile = PATH_TEMPLATES.TEMPLATE_FOLDER.'/languages/'.$language.'/templatetext.php';
		
		if (file_exists($languagefile))
		{			
			include(PATH_TEMPLATES.TEMPLATE_FOLDER.'/languages/'.$language.'/templatetext.php');
			
			foreach ($tt as $key => $item)
			{
				$tt[$key] = parseForeignLanguageCharacters($item);
			}
			
			$templatetext = $tt;
		}
		else
		{
			$templatetext = array();
		}
	}
	
	function gTT($key)
	{
		return getTemplateText($key);
	}
	
	function getTemplateText($key)
	{
		global $templatetext;
		
		return ($templatetext[$key]);
	}
	
	// Loads a template file, right now it's just a require but in the future it will look to parent template(s) if necessary
	// A similar function will need to be available for controllers to access the main template files
	if(!function_exists('loadTemplateElement'))
	{
		function loadTemplateElement($type, $name, $variables = array())
		{
			foreach ($variables as $key => $item)
			{
				# Defined here so they are in scope
				$$key = $item;
			}
		
			require pickTemplateFile("elements/{$type}/{$type}_{$name}.php");
		}
	}
	
	// And here that similar function is
	// That whole variables thing is going to be a hassle again
	function pickTemplateFile($name)
	{
		global $template_family; # It's just so much easier to have this this way
		foreach($template_family as $template)
		{
			$file = PATH_TEMPLATES.$template.'/'.$name;
			if(file_exists($file))
				return $file;
		}
		# Give up if nothing is found, I guess, but it's bad if this is ever used
		return PATH_TEMPLATES.TEMPLATE_FOLDER.'/'.$name;
	}
	
	function buildTemplateFamily()
	{
		$template_family = array();
		$parent = TEMPLATE_FOLDER;
		while($parent)
		{
			$template_family[] = $parent;
			include PATH_TEMPLATES.$parent.'/config.php';
			$parent = $template['meta']['parent'];
			if(!$parent || in_array($parent, $template_family)) break;
		}
		return $template_family;
	}
	
?>
